# RUN: %PYTHON %s | FileCheck %s

from mlir.ir import *
from mlir.dialects import pdl


def run(f):
    print("\nTEST:", f.__name__)
    f()
    return f


# CHECK-LABEL: TEST: test_attribute_type
@run
def test_attribute_type():
    with Context():
        parsedType = Type.parse("!pdl.attribute")
        constructedType = pdl.AttributeType.get()

        assert isinstance(parsedType, pdl.AttributeType)
        assert not isinstance(parsedType, pdl.OperationType)
        assert not isinstance(parsedType, pdl.RangeType)
        assert not isinstance(parsedType, pdl.TypeType)
        assert not isinstance(parsedType, pdl.ValueType)

        assert isinstance(constructedType, pdl.AttributeType)
        assert not isinstance(constructedType, pdl.OperationType)
        assert not isinstance(constructedType, pdl.RangeType)
        assert not isinstance(constructedType, pdl.TypeType)
        assert not isinstance(constructedType, pdl.ValueType)

        assert parsedType == constructedType

        # CHECK: !pdl.attribute
        print(parsedType)
        # CHECK: !pdl.attribute
        print(constructedType)


# CHECK-LABEL: TEST: test_operation_type
@run
def test_operation_type():
    with Context():
        parsedType = Type.parse("!pdl.operation")
        constructedType = pdl.OperationType.get()

        assert not isinstance(parsedType, pdl.AttributeType)
        assert isinstance(parsedType, pdl.OperationType)
        assert not isinstance(parsedType, pdl.RangeType)
        assert not isinstance(parsedType, pdl.TypeType)
        assert not isinstance(parsedType, pdl.ValueType)

        assert not isinstance(constructedType, pdl.AttributeType)
        assert isinstance(constructedType, pdl.OperationType)
        assert not isinstance(constructedType, pdl.RangeType)
        assert not isinstance(constructedType, pdl.TypeType)
        assert not isinstance(constructedType, pdl.ValueType)

        assert parsedType == constructedType

        # CHECK: !pdl.operation
        print(parsedType)
        # CHECK: !pdl.operation
        print(constructedType)


# CHECK-LABEL: TEST: test_range_type
@run
def test_range_type():
    with Context():
        typeType = Type.parse("!pdl.type")
        parsedType = Type.parse("!pdl.range<type>")
        constructedType = pdl.RangeType.get(typeType)
        elementType = constructedType.element_type

        assert not isinstance(parsedType, pdl.AttributeType)
        assert not isinstance(parsedType, pdl.OperationType)
        assert isinstance(parsedType, pdl.RangeType)
        assert not isinstance(parsedType, pdl.TypeType)
        assert not isinstance(parsedType, pdl.ValueType)

        assert not isinstance(constructedType, pdl.AttributeType)
        assert not isinstance(constructedType, pdl.OperationType)
        assert isinstance(constructedType, pdl.RangeType)
        assert not isinstance(constructedType, pdl.TypeType)
        assert not isinstance(constructedType, pdl.ValueType)

        assert parsedType == constructedType
        assert elementType == typeType

        # CHECK: !pdl.range<type>
        print(parsedType)
        # CHECK: !pdl.range<type>
        print(constructedType)
        # CHECK: !pdl.type
        print(elementType)


# CHECK-LABEL: TEST: test_type_type
@run
def test_type_type():
    with Context():
        parsedType = Type.parse("!pdl.type")
        constructedType = pdl.TypeType.get()

        assert not isinstance(parsedType, pdl.AttributeType)
        assert not isinstance(parsedType, pdl.OperationType)
        assert not isinstance(parsedType, pdl.RangeType)
        assert isinstance(parsedType, pdl.TypeType)
        assert not isinstance(parsedType, pdl.ValueType)

        assert not isinstance(constructedType, pdl.AttributeType)
        assert not isinstance(constructedType, pdl.OperationType)
        assert not isinstance(constructedType, pdl.RangeType)
        assert isinstance(constructedType, pdl.TypeType)
        assert not isinstance(constructedType, pdl.ValueType)

        assert parsedType == constructedType

        # CHECK: !pdl.type
        print(parsedType)
        # CHECK: !pdl.type
        print(constructedType)


# CHECK-LABEL: TEST: test_value_type
@run
def test_value_type():
    with Context():
        parsedType = Type.parse("!pdl.value")
        constructedType = pdl.ValueType.get()

        assert not isinstance(parsedType, pdl.AttributeType)
        assert not isinstance(parsedType, pdl.OperationType)
        assert not isinstance(parsedType, pdl.RangeType)
        assert not isinstance(parsedType, pdl.TypeType)
        assert isinstance(parsedType, pdl.ValueType)

        assert not isinstance(constructedType, pdl.AttributeType)
        assert not isinstance(constructedType, pdl.OperationType)
        assert not isinstance(constructedType, pdl.RangeType)
        assert not isinstance(constructedType, pdl.TypeType)
        assert isinstance(constructedType, pdl.ValueType)

        assert parsedType == constructedType

        # CHECK: !pdl.value
        print(parsedType)
        # CHECK: !pdl.value
        print(constructedType)


# CHECK-LABEL: TEST: test_type_without_context
@run
def test_type_without_context():
    # Constructing a type without the surrounding ir.Context context manager
    # should raise an exception but not crash.
    try:
        constructedType = pdl.ValueType.get()
    except RuntimeError as e:
        assert (
            "An MLIR function requires a Context but none was provided in the call or from the surrounding environment"
            in e.args[0]
        )
    else:
        assert False, "Expected TypeError to be raised."



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class InvalidStdVariantDataFormatterTestCase(TestBase):
    @add_test_categories(["libstdcxx"])
    def test(self):
        """Test STL data formatters for std::variant with invalid index."""
        self.build()

        (_, _, thread, _) = lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.expect(
            "frame variable v1",
            substrs=["v1 =  Active Type = char  {", "Value = 'x'", "}"],
        )

        var_v1 = thread.frames[0].FindVariable("v1")
        var_v1_raw_obj = var_v1.GetNonSyntheticValue()
        index_obj = var_v1_raw_obj.GetChildMemberWithName("_M_index")
        self.assertTrue(index_obj and index_obj.IsValid())

        INVALID_INDEX = "100"
        index_obj.SetValueFromCString(INVALID_INDEX)

        self.expect("frame variable v1", substrs=["v1 =  <Invalid>"])



import lldb


class OperatingSystemPlugIn(object):
    """Class that provides data for an instance of a LLDB 'OperatingSystemPython' plug-in class
    This version stops once with threads 0x111 and 0x222, then stops a second time with threads
    0x111 and 0x333."""

    def __init__(self, process):
        """Initialization needs a valid.SBProcess object.

        This plug-in will get created after a live process is valid and has stopped for the first time.
        """
        self.process = None
        self.registers = None
        self.threads = None
        self.times_called = 0
        if isinstance(process, lldb.SBProcess) and process.IsValid():
            self.process = process
            self.threads = None  # Will be an dictionary containing info for each thread

    def get_target(self):
        return self.process.target

    def get_thread_info(self):
        self.times_called += 1

        if self.times_called == 1:
            self.threads = [
                {
                    "tid": 0x111,
                    "name": "one",
                    "queue": "queue1",
                    "state": "stopped",
                    "stop_reason": "none",
                    "core": 1,
                },
                {
                    "tid": 0x222,
                    "name": "two",
                    "queue": "queue2",
                    "state": "stopped",
                    "stop_reason": "none",
                    "core": 0,
                },
            ]
        else:
            self.threads = [
                {
                    "tid": 0x111,
                    "name": "one",
                    "queue": "queue1",
                    "state": "stopped",
                    "stop_reason": "none",
                    "core": 1,
                },
                {
                    "tid": 0x333,
                    "name": "three",
                    "queue": "queue3",
                    "state": "stopped",
                    "stop_reason": "none",
                    "core": 0,
                },
            ]
        return self.threads



"""
Test number of threads.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExitDuringStepTestCase(TestBase):
    @skipIfWindows  # This is flakey on Windows: llvm.org/pr38373
    def test(self):
        """Test thread exit during step handling."""
        self.build()
        self.exit_during_step_base(
            "thread step-inst -m all-threads", "stop reason = instruction step", True
        )

    @skipIfWindows  # This is flakey on Windows: llvm.org/pr38373
    def test_step_over(self):
        """Test thread exit during step-over handling."""
        self.build()
        self.exit_during_step_base(
            "thread step-over -m all-threads", "stop reason = step over", False
        )

    @skipIfWindows  # This is flakey on Windows: llvm.org/pr38373
    def test_step_in(self):
        """Test thread exit during step-in handling."""
        self.build()
        self.exit_during_step_base(
            "thread step-in -m all-threads", "stop reason = step in", False
        )

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line numbers to break and continue.
        self.breakpoint = line_number("main.cpp", "// Set breakpoint here")
        self.continuepoint = line_number("main.cpp", "// Continue from here")

    def exit_during_step_base(self, step_cmd, step_stop_reason, by_instruction):
        """Test thread exit during step handling."""
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # This should create a breakpoint in the main thread.
        self.bp_num = lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.breakpoint, num_expected_locations=1
        )

        # The breakpoint list should show 1 location.
        self.expect(
            "breakpoint list -f",
            "Breakpoint location shown correctly",
            substrs=[
                "1: file = 'main.cpp', line = %d, exact_match = 0, locations = 1"
                % self.breakpoint
            ],
        )

        # Run the program.
        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # Get the target process
        target = self.dbg.GetSelectedTarget()
        process = target.GetProcess()

        num_threads = process.GetNumThreads()
        # Make sure we see all three threads
        self.assertGreaterEqual(
            num_threads,
            3,
            "Number of expected threads and actual threads do not match.",
        )

        stepping_thread = lldbutil.get_one_thread_stopped_at_breakpoint_id(
            process, self.bp_num
        )
        self.assertIsNotNone(
            stepping_thread, "Could not find a thread stopped at the breakpoint"
        )

        current_line = self.breakpoint
        stepping_frame = stepping_thread.GetFrameAtIndex(0)
        self.assertEqual(
            current_line,
            stepping_frame.GetLineEntry().GetLine(),
            "Starting line for stepping doesn't match breakpoint line.",
        )

        # Keep stepping until we've reached our designated continue point
        while current_line != self.continuepoint:
            # Since we're using the command interpreter to issue the thread command
            # (on the selected thread) we need to ensure the selected thread is the
            # stepping thread.
            if stepping_thread != process.GetSelectedThread():
                process.SetSelectedThread(stepping_thread)

            self.runCmd(step_cmd)

            frame = stepping_thread.GetFrameAtIndex(0)

            current_line = frame.GetLineEntry().GetLine()

            if by_instruction and current_line == 0:
                continue

            self.assertGreaterEqual(
                current_line,
                self.breakpoint,
                "Stepped to unexpected line, " + str(current_line),
            )
            self.assertLessEqual(
                current_line,
                self.continuepoint,
                "Stepped to unexpected line, " + str(current_line),
            )

        self.runCmd("thread list")

        # Update the number of threads
        new_num_threads = process.GetNumThreads()

        # Check to see that we reduced the number of threads as expected
        self.assertEqual(
            new_num_threads,
            num_threads - 1,
            "Number of threads did not reduce by 1 after thread exit.",
        )

        self.expect(
            "thread list",
            "Process state is stopped due to step",
            substrs=["stopped", step_stop_reason],
        )

        # Run to completion
        self.runCmd("continue")

        # At this point, the inferior process should have exited.
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)



"""
Test some SBValue APIs.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ChangeValueAPITestCase(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # We'll use the test method name as the exe_name.
        self.exe_name = self.testMethodName
        # Find the line number to of function 'c'.
        self.line = line_number("main.c", "// Stop here and set values")
        self.check_line = line_number("main.c", "// Stop here and check values")
        self.end_line = line_number("main.c", "// Set a breakpoint here at the end")

    @expectedFlakeyLinux("llvm.org/pr25652")
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24772")
    def test_change_value(self):
        """Exercise the SBValue::SetValueFromCString API."""
        d = {"EXE": self.exe_name}
        self.build(dictionary=d)
        self.setTearDownCleanup(dictionary=d)
        exe = self.getBuildArtifact(self.exe_name)

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create the breakpoint inside function 'main'.
        breakpoint = target.BreakpointCreateByLocation("main.c", self.line)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Create the breakpoint inside the function 'main'
        check_breakpoint = target.BreakpointCreateByLocation("main.c", self.check_line)
        self.assertTrue(check_breakpoint, VALID_BREAKPOINT)

        # Create the breakpoint inside function 'main'.
        end_breakpoint = target.BreakpointCreateByLocation("main.c", self.end_line)
        self.assertTrue(end_breakpoint, VALID_BREAKPOINT)

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Get Frame #0.
        self.assertState(process.GetState(), lldb.eStateStopped)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition",
        )
        frame0 = thread.GetFrameAtIndex(0)
        self.assertTrue(frame0.IsValid(), "Got a valid frame.")

        # Get the val variable and change it:
        error = lldb.SBError()

        val_value = frame0.FindVariable("val")
        self.assertTrue(val_value.IsValid(), "Got the SBValue for val")
        actual_value = val_value.GetValueAsSigned(error, 0)
        self.assertSuccess(error, "Got a value from val")
        self.assertEqual(actual_value, 100, "Got the right value from val")

        result = val_value.SetValueFromCString("12345")
        self.assertTrue(result, "Setting val returned True.")
        actual_value = val_value.GetValueAsSigned(error, 0)
        self.assertSuccess(error, "Got a changed value from val")
        self.assertEqual(actual_value, 12345, "Got the right changed value from val")

        # Now check that we can set a structure element:

        mine_value = frame0.FindVariable("mine")
        self.assertTrue(mine_value.IsValid(), "Got the SBValue for mine")

        mine_second_value = mine_value.GetChildMemberWithName("second_val")
        self.assertTrue(mine_second_value.IsValid(), "Got second_val from mine")
        actual_value = mine_second_value.GetValueAsUnsigned(error, 0)
        self.assertTrue(error.Success(), "Got an unsigned value for second_val")
        self.assertEqual(actual_value, 5555)

        result = mine_second_value.SetValueFromCString("98765")
        self.assertTrue(result, "Success setting mine.second_value.")
        actual_value = mine_second_value.GetValueAsSigned(error, 0)
        self.assertTrue(error.Success(), "Got a changed value from mine.second_val")
        self.assertEqual(
            actual_value, 98765, "Got the right changed value from mine.second_val"
        )

        # Next do the same thing with the pointer version.
        ptr_value = frame0.FindVariable("ptr")
        self.assertTrue(ptr_value.IsValid(), "Got the SBValue for ptr")

        ptr_second_value = ptr_value.GetChildMemberWithName("second_val")
        self.assertTrue(ptr_second_value.IsValid(), "Got second_val from ptr")
        actual_value = ptr_second_value.GetValueAsUnsigned(error, 0)
        self.assertTrue(error.Success(), "Got an unsigned value for ptr->second_val")
        self.assertEqual(actual_value, 6666)

        result = ptr_second_value.SetValueFromCString("98765")
        self.assertTrue(result, "Success setting ptr->second_value.")
        actual_value = ptr_second_value.GetValueAsSigned(error, 0)
        self.assertTrue(error.Success(), "Got a changed value from ptr->second_val")
        self.assertEqual(
            actual_value, 98765, "Got the right changed value from ptr->second_val"
        )

        ptr_fourth_value = ptr_value.GetChildMemberWithName("fourth_val")
        self.assertTrue(ptr_fourth_value.IsValid(), "Got fourth_val from ptr")
        fourth_actual_value = ptr_fourth_value.GetValueAsUnsigned(error, 1)
        self.assertTrue(error.Success(), "Got an unsigned value for ptr->fourth_val")
        self.assertEqual(fourth_actual_value, 0)

        result = ptr_fourth_value.SetValueFromCString("true")
        self.assertTrue(result, "Success setting ptr->fourth_val.")
        fourth_actual_value = ptr_fourth_value.GetValueAsSigned(error, 0)
        self.assertTrue(error.Success(), "Got a changed value from ptr->fourth_val")
        self.assertEqual(
            fourth_actual_value, 1, "Got the right changed value from ptr->fourth_val"
        )

        result = ptr_fourth_value.SetValueFromCString("NO")
        self.assertFalse(result, "Failed setting ptr->fourth_val.")
        fourth_actual_value = ptr_fourth_value.GetValueAsSigned(error, 0)
        self.assertTrue(error.Success(), "Got the original value from ptr->fourth_val")
        self.assertEqual(
            fourth_actual_value,
            1,
            "Got the original changed value from ptr->fourth_val",
        )

        # gcc may set multiple locations for breakpoint
        breakpoint.SetEnabled(False)

        # Now continue.
        process.Continue()

        self.assertState(process.GetState(), lldb.eStateStopped)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(),
            "There should be a thread stopped due to breakpoint condition",
        )

        expected_value = (
            "Val - 12345 Mine - 55, 98765, 55555555, 0. Ptr - 66, 98765, 66666666, 1"
        )
        stdout = process.GetSTDOUT(1000)
        self.assertIn(expected_value, stdout, "STDOUT showed changed values.")

        # Finally, change the stack pointer to 0, and we should not make it to
        # our end breakpoint.
        frame0 = thread.GetFrameAtIndex(0)
        self.assertTrue(frame0.IsValid(), "Second time: got a valid frame.")
        sp_value = frame0.FindValue("sp", lldb.eValueTypeRegister)
        self.assertTrue(sp_value.IsValid(), "Got a stack pointer value")
        result = sp_value.SetValueFromCString("1")
        self.assertTrue(result, "Setting sp returned true.")
        actual_value = sp_value.GetValueAsUnsigned(error, 0)
        self.assertSuccess(error, "Got a changed value for sp")
        self.assertEqual(actual_value, 1, "Got the right changed value for sp.")

        # Boundary condition test the SBValue.CreateValueFromExpression() API.
        # LLDB should not crash!
        nosuchval = mine_value.CreateValueFromExpression(None, None)

        process.Continue()

        self.assertState(process.GetState(), lldb.eStateStopped)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertIsNone(
            thread,
            "We should not have managed to hit our second breakpoint with sp == 1",
        )

        process.Kill()



"""
Use lldb Python API to test dynamic values in ObjC
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCDynamicValueTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.

        self.source_name = "dynamic-value.m"
        self.set_property_line = line_number(
            self.source_name,
            "// This is the line in setProperty, make sure we step to here.",
        )
        self.handle_SourceBase = line_number(
            self.source_name, "// Break here to check dynamic values."
        )
        self.main_before_setProperty_line = line_number(
            self.source_name, "// Break here to see if we can step into real method."
        )

    @add_test_categories(["pyapi"])
    @expectedFailureDarwin("llvm.org/pr20271 rdar://18684107")
    def test_get_objc_dynamic_vals(self):
        """Test fetching ObjC dynamic values."""
        if self.getArchitecture() == "i386":
            # rdar://problem/9946499
            self.skipTest("Dynamic types for ObjC V1 runtime not implemented")

        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoints:

        handle_SourceBase_bkpt = target.BreakpointCreateByLocation(
            self.source_name, self.handle_SourceBase
        )
        self.assertTrue(
            handle_SourceBase_bkpt and handle_SourceBase_bkpt.GetNumLocations() == 1,
            VALID_BREAKPOINT,
        )

        main_before_setProperty_bkpt = target.BreakpointCreateByLocation(
            self.source_name, self.main_before_setProperty_line
        )
        self.assertTrue(
            main_before_setProperty_bkpt
            and main_before_setProperty_bkpt.GetNumLocations() == 1,
            VALID_BREAKPOINT,
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        threads = lldbutil.get_threads_stopped_at_breakpoint(
            process, main_before_setProperty_bkpt
        )
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        #
        #  At this point, myObserver has a Source pointer that is actually a KVO swizzled SourceDerived
        #  make sure we can get that properly:

        frame = thread.GetFrameAtIndex(0)
        myObserver = frame.FindVariable("myObserver", lldb.eDynamicCanRunTarget)
        self.assertTrue(myObserver)
        myObserver_source = myObserver.GetChildMemberWithName(
            "_source", lldb.eDynamicCanRunTarget
        )
        self.examine_SourceDerived_ptr(myObserver_source)

        #
        # Make sure a static value can be correctly turned into a dynamic
        # value.

        frame = thread.GetFrameAtIndex(0)
        myObserver_static = frame.FindVariable("myObserver", lldb.eNoDynamicValues)
        self.assertTrue(myObserver_static)
        myObserver = myObserver_static.GetDynamicValue(lldb.eDynamicCanRunTarget)
        myObserver_source = myObserver.GetChildMemberWithName(
            "_source", lldb.eDynamicCanRunTarget
        )
        self.examine_SourceDerived_ptr(myObserver_source)

        # The "frame var" code uses another path to get into children, so let's
        # make sure that works as well:

        result = lldb.SBCommandReturnObject()

        self.expect(
            "frame var -d run-target myObserver->_source",
            "frame var finds its way into a child member",
            patterns=[r"\(SourceDerived \*\)"],
        )

        # check that our ObjC GetISA() does a good job at hiding KVO swizzled
        # classes

        self.expect(
            "frame var -d run-target myObserver->_source -T",
            "the KVO-ed class is hidden",
            substrs=["SourceDerived"],
        )

        self.expect(
            "frame var -d run-target myObserver->_source -T",
            "the KVO-ed class is hidden",
            matching=False,
            substrs=["NSKVONotify"],
        )

        # This test is not entirely related to the main thrust of this test case, but since we're here,
        # try stepping into setProperty, and make sure we get into the version
        # in Source:

        thread.StepInto()

        threads = lldbutil.get_stopped_threads(process, lldb.eStopReasonPlanComplete)
        self.assertEqual(len(threads), 1)
        line_entry = threads[0].GetFrameAtIndex(0).GetLineEntry()

        self.assertEqual(line_entry.GetLine(), self.set_property_line)
        self.assertEqual(line_entry.GetFileSpec().GetFilename(), self.source_name)

        # Okay, back to the main business.  Continue to the handle_SourceBase
        # and make sure we get the correct dynamic value.

        threads = lldbutil.continue_to_breakpoint(process, handle_SourceBase_bkpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)

        # Get "object" using FindVariable:

        noDynamic = lldb.eNoDynamicValues
        useDynamic = lldb.eDynamicCanRunTarget

        object_static = frame.FindVariable("object", noDynamic)
        object_dynamic = frame.FindVariable("object", useDynamic)

        # Delete this object to make sure that this doesn't cause havoc with
        # the dynamic object that depends on it.
        del object_static

        self.examine_SourceDerived_ptr(object_dynamic)

        # Get "this" using FindValue, make sure that works too:
        object_static = frame.FindValue(
            "object", lldb.eValueTypeVariableArgument, noDynamic
        )
        object_dynamic = frame.FindValue(
            "object", lldb.eValueTypeVariableArgument, useDynamic
        )
        del object_static
        self.examine_SourceDerived_ptr(object_dynamic)

        # Get "this" using the EvaluateExpression:
        object_static = frame.EvaluateExpression("object", noDynamic)
        object_dynamic = frame.EvaluateExpression("object", useDynamic)
        del object_static
        self.examine_SourceDerived_ptr(object_dynamic)

        # Continue again to the handle_SourceBase and make sure we get the correct dynamic value.
        # This one looks exactly the same, but in fact this is an "un-KVO'ed" version of SourceBase, so
        # its isa pointer points to SourceBase not NSKVOSourceBase or
        # whatever...

        threads = lldbutil.continue_to_breakpoint(process, handle_SourceBase_bkpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        frame = thread.GetFrameAtIndex(0)

        # Get "object" using FindVariable:

        object_static = frame.FindVariable("object", noDynamic)
        object_dynamic = frame.FindVariable("object", useDynamic)

        # Delete this object to make sure that this doesn't cause havoc with
        # the dynamic object that depends on it.
        del object_static

        self.examine_SourceDerived_ptr(object_dynamic)

    def examine_SourceDerived_ptr(self, object):
        self.assertTrue(object)
        self.assertNotEqual(object.GetTypeName().find("SourceDerived"), -1)
        derivedValue = object.GetChildMemberWithName("_derivedValue")
        self.assertTrue(derivedValue)
        self.assertEqual(int(derivedValue.GetValue(), 0), 30)



"""
Test that we work properly with classes with the trivial_abi attribute
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestTrivialABI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessSupportedTypeAttribute("trivial_abi")
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr37995")
    @expectedFailureAll(
        archs=["aarch64"], oslist=["freebsd", "linux"], bugnumber="llvm.org/pr44161"
    )
    def test_call_trivial(self):
        """Test that we can print a variable & call a function with a trivial ABI class."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.expr_test(True)

    @skipUnlessSupportedTypeAttribute("trivial_abi")
    # fixed for SysV-x86_64 ABI, but not Windows-x86_64
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr36870")
    @expectedFailureAll(
        archs=["arm$", "aarch64"],
        oslist=["freebsd", "linux"],
        bugnumber="llvm.org/pr44161",
    )
    @expectedFailureAll(
        archs=["arm64", "arm64e"], bugnumber="<rdar://problem/57844240>"
    )
    def test_call_nontrivial(self):
        """Test that we can print a variable & call a function on the same class w/o the trivial ABI marker."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.cpp")
        self.expr_test(False)

    def check_value(self, test_var, ivar_value):
        self.assertSuccess(test_var.GetError(), "Invalid valobj")
        ivar = test_var.GetChildMemberWithName("ivar")
        self.assertSuccess(test_var.GetError(), "Failed to fetch ivar")
        self.assertEqual(
            ivar_value, ivar.GetValueAsSigned(), "Got the right value for ivar"
        )

    def check_frame(self, thread):
        frame = thread.frames[0]
        inVal_var = frame.FindVariable("inVal")
        self.check_value(inVal_var, 10)

        options = lldb.SBExpressionOptions()
        inVal_expr = frame.EvaluateExpression("inVal", options)
        self.check_value(inVal_expr, 10)

        thread.StepOut()
        outVal_ret = thread.GetStopReturnValue()
        self.check_value(outVal_ret, 30)

    def expr_test(self, trivial):
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint here", self.main_source_file
        )

        # Stop in a function that takes a trivial value, and try both frame var & expr to get its value:
        if trivial:
            self.check_frame(thread)
            return

        # Now continue to the same thing without the trivial_abi and see if we get that right:
        threads = lldbutil.continue_to_breakpoint(process, bkpt)
        self.assertEqual(len(threads), 1, "Hit my breakpoint the second time.")

        self.check_frame(threads[0])



"""
Tests that LLDB can correctly set up a disassembler using extensions from the .riscv.attributes section.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os


class TestDisassembler(TestBase):
    expected_zbb_instrs = ["andn", "orn", "xnor", "rol", "ror"]

    @skipIfLLVMTargetMissing("RISCV")
    def test_without_riscv_attributes(self):
        """
        Tests disassembly of a riscv binary without the .riscv.attributes.
        Without the .riscv.attributes section lldb won't set up a disassembler to
        handle the bitmanip extension, so it is not expected to see zbb instructions
        in the output.
        """
        yaml = os.path.join(self.getSourceDir(), "stripped.out.yaml")
        exe = self.getBuildArtifact("stripped.out")
        self.yaml2obj(yaml, exe)

        target = self.dbg.CreateTarget(exe)

        self.expect("disassemble --name do_zbb_stuff")
        output = self.res.GetOutput()

        for instr in self.expected_zbb_instrs:
            self.assertFalse(
                instr in output, "Zbb instructions should not be disassembled"
            )

        self.assertEqual(
            output.count("unknown"),
            len(self.expected_zbb_instrs),
            "Instructions from the Zbb extension should be displayed as <unknown>",
        )

    @skipIfLLVMTargetMissing("RISCV")
    def test_with_riscv_attributes(self):
        """
        Tests disassembly of a riscv binary with the .riscv.attributes.
        """
        yaml = os.path.join(self.getSourceDir(), "a.out.yaml")
        exe = self.getBuildArtifact("a.out")
        self.yaml2obj(yaml, exe)

        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))

        self.expect("disassemble --name do_zbb_stuff")
        output = self.res.GetOutput()

        for instr in self.expected_zbb_instrs:
            self.assertTrue(instr in output, "Invalid disassembler output")

    @skipIfLLVMTargetMissing("RISCV")
    def test_conflicting_extensions(self):
        """
        This test demonstrates the scenario where:
        1. file_with_zcd.c is compiled with rv64gc (includes C and D).
        2. file_with_zcmp.c is compiled with rv64imad_zcmp (includes Zcmp).
        3. The linker merges .riscv.attributes, creating the union: C + D + Zcmp.

        The Zcmp extension is incompatible with the C extension when the D extension is enabled.
        Therefore, the arch string contains conflicting extensions, and LLDB should
        display an appropriate warning in this case.
        """
        yaml = os.path.join(self.getSourceDir(), "conflicting.out.yaml")
        exe = self.getBuildArtifact("conflicting.out")
        self.yaml2obj(yaml, exe)

        target = self.dbg.CreateTarget(self.getBuildArtifact("a.out"))
        output = self.res.GetOutput()

        self.assertIn(
            output,
            "The .riscv.attributes section contains an invalid RISC-V arch string",
        )



"""
Test some lldb command abbreviations.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FatArchiveTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipUnlessDarwin
    def test_breakpoint_resolution_dwarf(self):
        if self.getArchitecture() == "x86_64":
            self.build()
            self.main()
        else:
            self.skipTest(
                "This test requires x86_64 as the architecture for the inferior"
            )

    def main(self):
        """This test compiles a quick example by making a fat file (universal) full of
        skinny .o files and makes sure we can use them to resolve breakpoints when doing
        DWARF in .o file debugging. The only thing this test needs to do is to compile and
        set a breakpoint in the target and verify any breakpoint locations have valid debug
        info for the function, and source file and line."""
        exe = self.getBuildArtifact("a.out")

        # Create the target
        target = self.dbg.CreateTarget(exe)

        # Create a breakpoint by name
        breakpoint = target.BreakpointCreateByName("foo", exe)
        self.assertTrue(breakpoint, VALID_BREAKPOINT)

        # Make sure the breakpoint resolves to a function, file and line
        for bp_loc in breakpoint:
            # Get a section offset address (lldb.SBAddress) from the breakpoint
            # location
            bp_loc_addr = bp_loc.GetAddress()
            line_entry = bp_loc_addr.GetLineEntry()
            function = bp_loc_addr.GetFunction()
            self.assertTrue(
                function.IsValid(),
                "Verify breakpoint in fat BSD archive has valid function debug info",
            )
            self.assertTrue(
                line_entry.GetFileSpec(),
                "Verify breakpoint in fat BSD archive has source file information",
            )
            self.assertNotEqual(
                line_entry.GetLine(),
                0,
                "Verify breakpoint in fat BSD archive has source line information",
            )



"""
Test the 'memory find' command.
"""


import lldb
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *


class MemoryFindTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.line = line_number("main.cpp", "// break here")

    def test_memory_find(self):
        """Test the 'memory find' command."""
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)

        # Break in main() after the variables are assigned values.
        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # The stop reason of the thread should be breakpoint.
        self.expect(
            "thread list",
            STOPPED_DUE_TO_BREAKPOINT,
            substrs=["stopped", "stop reason = breakpoint"],
        )

        # The breakpoint should have a hit count of 1.
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # Test the memory find commands.

        # Empty search string should be handled.
        self.expect(
            'memory find -s "" `stringdata` `stringdata+16`',
            error=True,
            substrs=["error: search string must have non-zero length."],
        )

        self.expect(
            'memory find -s "in const" `stringdata` `stringdata+(int)strlen(stringdata)`',
            substrs=["data found at location: 0x", "69 6e 20 63", "in const"],
        )

        # Invalid expr is an error.
        self.expect(
            'memory find -e "not_a_symbol" `&bytedata[0]` `&bytedata[15]`',
            substrs=[
                "Expression evaluation failed:",
                "use of undeclared identifier 'not_a_symbol'",
            ],
            error=True,
        )

        self.expect(
            'memory find -e "" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Expression evaluation failed:",
                "No result returned from expression. Exit status: 1",
            ],
            error=True,
        )

        # Valid expressions/strings
        self.expect(
            'memory find -e "(uint8_t)0x22" `&bytedata[0]` `&bytedata[15]`',
            substrs=["data found at location: 0x", "22 33 44 55 66"],
        )

        self.expect(
            'memory find -e "(uint8_t)0x22" `&bytedata[0]` `&bytedata[2]`',
            substrs=["data not found within the range."],
        )

        self.expect(
            'memory find -s "nothere" `stringdata` `stringdata+5`',
            substrs=["data not found within the range."],
        )

        self.expect(
            'memory find -s "nothere" `stringdata` `stringdata+10`',
            substrs=["data not found within the range."],
        )

        # Expression results with unsupported result types.
        self.expect(
            'memory find -e "ThreeBytes{}" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Only expressions resulting in 1, 2, 4, or 8-byte-sized values are supported"
            ],
            error=True,
        )

        self.expect(
            'memory find -e "FiveBytes{}" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Only expressions resulting in 1, 2, 4, or 8-byte-sized values are supported"
            ],
            error=True,
        )

        self.expect(
            'memory find -e "SixBytes{}" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Only expressions resulting in 1, 2, 4, or 8-byte-sized values are supported"
            ],
            error=True,
        )

        self.expect(
            'memory find -e "SevenBytes{}" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Only expressions resulting in 1, 2, 4, or 8-byte-sized values are supported"
            ],
            error=True,
        )

        self.expect(
            'memory find -e "NineBytes{}" `&bytedata[0]` `&bytedata[2]`',
            substrs=[
                "Only expressions resulting in 1, 2, 4, or 8-byte-sized values are supported"
            ],
            error=True,
        )



"""
If there is a definition of a type in the current
Execution Context's CU, then we should use that type
even if there are other definitions of the type in other
CU's.  Assert that that is true.
"""


import lldb
import lldbsuite.test.lldbutil as lldbutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *


class TestUseClosestType(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @expectedFailureAll(bugnumber="<rdar://problem/53262085>")
    def test_use_in_expr(self):
        """Use the shadowed type directly, see if we get a conflicting type definition."""
        self.build()
        self.main_source_file = lldb.SBFileSpec("main.c")
        self.expr_test()

    def run_and_check_expr(self, num_children, child_type):
        frame = self.thread.GetFrameAtIndex(0)
        result = frame.EvaluateExpression(
            "struct Foo *$mine = (struct Foo *) malloc(sizeof(struct Foo)); $mine"
        )
        self.assertSuccess(
            result.GetError(),
            "Failed to parse an expression using a multiply defined type",
        )
        self.assertEqual(
            result.GetTypeName(), "struct Foo *", "The result has the right typename."
        )
        self.assertEqual(
            result.GetNumChildren(), num_children, "Got the right number of children"
        )
        self.assertEqual(
            result.GetChildAtIndex(0).GetTypeName(), child_type, "Got the right type."
        )

    def expr_test(self):
        """Run to a breakpoint in main.c, check that an expression referring to Foo gets the
        local three int version.  Then run to a breakpoint in other.c and check that an
        expression referring to Foo gets the two char* version."""

        (target, process, self.thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint in main", self.main_source_file
        )

        self.run_and_check_expr(3, "int")
        lldbutil.run_to_source_breakpoint(
            self, "Set a breakpoint in other", lldb.SBFileSpec("other.c")
        )
        self.run_and_check_expr(2, "char *")



"""
Test sending SIGINT to the embedded Python REPL.
"""

import os

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test.lldbpexpect import PExpectTest


class TestCase(PExpectTest):
    def start_python_repl(self):
        """Starts up the embedded Python REPL."""
        self.launch()
        # Start the embedded Python REPL via the 'script' command.
        self.child.send("script -l python --\n")
        # Wait for the Python REPL prompt.
        self.child.expect(">>>")

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    @skipIfWindows
    @skipIf(oslist=["linux"], archs=["arm$", "aarch64"])
    def test_while_evaluating_code(self):
        """Tests SIGINT handling while Python code is being evaluated."""
        self.start_python_repl()

        # Start a long-running command that we try to abort with SIGINT.
        # Note that we dont actually wait 10000s in this code as pexpect or
        # lit will kill the test way before that.
        self.child.send("import time; print('running' + 'now'); time.sleep(10000);\n")

        # Make sure the command is actually being evaluated at the moment by
        # looking at the string that the command is printing.
        # Don't check for a needle that also occurs in the program itself to
        # prevent that echoing will make this check pass unintentionally.
        self.child.expect("runningnow")

        # Send SIGINT to the LLDB process.
        self.child.sendintr()

        # This should get transformed to a KeyboardInterrupt which is the same
        # behaviour as the standalone Python REPL. It should also interrupt
        # the evaluation of our sleep statement.
        self.child.expect("KeyboardInterrupt")
        # Send EOF to quit the Python REPL.
        self.child.sendeof()

        self.quit()

    # PExpect uses many timeouts internally and doesn't play well
    # under ASAN on a loaded machine..
    @skipIfAsan
    # FIXME: On Linux the Python code that reads from stdin seems to block until
    # it has finished reading a line before handling any queued signals.
    @skipIf(hostoslist=["linux"])
    @skipIfWindows
    def test_while_waiting_on_input(self):
        """Tests SIGINT handling while the REPL is waiting on input from
        stdin."""
        self.start_python_repl()

        # Send SIGINT to the LLDB process.
        self.child.sendintr()
        # This should get transformed to a KeyboardInterrupt which is the same
        # behaviour as the standalone Python REPL.
        self.child.expect("KeyboardInterrupt")
        # Send EOF to quit the Python REPL.
        self.child.sendeof()

        self.quit()



import lldb


def some_command_here(debugger, command, result, d):
    if command == "a":
        print("Victory is mine", file=result)
        return True
    else:
        print("Sadness for all", file=result)
        return False



"""
Test some more expression commands.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ExprCommands2TestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.
        self.line = line_number(
            "main.cpp", "// Please test many expressions while stopped at this line:"
        )

    def test_more_expr_commands(self):
        """Test some more expression commands."""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=False
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # Does static casting work?
        self.expect("expression (int*)argv", startstr="(int *) $0 = 0x")
        # (int *) $0 = 0x00007fff5fbff258

        # Do return values containing the contents of expression locals work?
        self.expect("expression int i = 5; i", startstr="(int) $1 = 5")
        # (int) $2 = 5
        self.expect("expression $1 + 1", startstr="(int) $2 = 6")
        # (int) $3 = 6

        # Do return values containing the results of static expressions work?
        self.expect("expression 20 + 3", startstr="(int) $3 = 23")
        # (int) $4 = 5
        self.expect("expression $3 + 1", startstr="(int) $4 = 24")
        # (int) $5 = 6

    @skipIfLinux
    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24489")
    def test_expr_symbols(self):
        """Test symbols."""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=False
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # Do anonymous symbols work?
        self.expect("expression ((char**)environ)[0]", startstr="(char *) $0 = 0x")
        # (char *) $1 = 0x00007fff5fbff298 "Apple_PubSub_Socket_Render=/tmp/launch-7AEsUD/Render"



"""Test stepping over watchpoints and instruction stepping past watchpoints."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestStepOverWatchpoint(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def get_to_start(self, bkpt_text):
        """Test stepping over watchpoints and instruction stepping past watchpoints.."""
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, bkpt_text, lldb.SBFileSpec("main.c")
        )
        return (target, process, thread, frame, read_watchpoint)

    @add_test_categories(["basic_process"])
    # kernel disables wp's over instruction step, fixed in macOS 14.4.
    @skipIf(macos_version=["<", "14.4"])
    def test_step_over_read_watchpoint(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here for read watchpoints", lldb.SBFileSpec("main.c")
        )

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame.IsValid(), "Failed to get frame.")

        read_value = frame.FindValue("g_watch_me_read", lldb.eValueTypeVariableGlobal)
        self.assertTrue(read_value.IsValid(), "Failed to find read value.")

        error = lldb.SBError()

        # resolve_location=True, read=True, write=False
        read_watchpoint = read_value.Watch(True, True, False, error)
        self.assertSuccess(error, "Error while setting watchpoint")
        self.assertTrue(read_watchpoint, "Failed to set read watchpoint.")

        # Disable the breakpoint we hit so we don't muddy the waters with
        # stepping off from the breakpoint:
        bkpt.SetEnabled(False)

        thread.StepOver()
        self.assertStopReason(
            thread.GetStopReason(),
            lldb.eStopReasonWatchpoint,
            STOPPED_DUE_TO_WATCHPOINT,
        )
        self.assertEqual(thread.stop_description, "watchpoint 1")

        process.Continue()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        self.assertEqual(thread.stop_description, "step over")

        self.step_inst_for_watchpoint(1)

    @add_test_categories(["basic_process"])
    # kernel disables wp's over instruction step, fixed in macOS 14.4.
    @skipIf(macos_version=["<", "14.4"])
    def test_step_over_write_watchpoint(self):
        self.build()
        target, process, thread, bkpt = lldbutil.run_to_source_breakpoint(
            self, "break here for modify watchpoints", lldb.SBFileSpec("main.c")
        )

        # Disable the breakpoint we hit so we don't muddy the waters with
        # stepping off from the breakpoint:
        bkpt.SetEnabled(False)

        frame = thread.GetFrameAtIndex(0)
        self.assertTrue(frame.IsValid(), "Failed to get frame.")

        write_value = frame.FindValue("g_watch_me_write", lldb.eValueTypeVariableGlobal)
        self.assertTrue(write_value, "Failed to find write value.")

        error = lldb.SBError()
        # resolve_location=True, read=False, modify=True
        write_watchpoint = write_value.Watch(True, False, True, error)
        self.assertTrue(write_watchpoint, "Failed to set write watchpoint.")
        self.assertSuccess(error, "Error while setting watchpoint")

        thread.StepOver()
        self.assertStopReason(
            thread.GetStopReason(),
            lldb.eStopReasonWatchpoint,
            STOPPED_DUE_TO_WATCHPOINT,
        )
        self.assertEqual(thread.stop_description, "watchpoint 1")

        process.Continue()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        self.assertEqual(thread.stop_description, "step over")

        self.step_inst_for_watchpoint(1)

    def step_inst_for_watchpoint(self, wp_id):
        watchpoint_hit = False
        current_line = self.frame().GetLineEntry().GetLine()
        while self.frame().GetLineEntry().GetLine() == current_line:
            self.thread().StepInstruction(False)  # step_over=False
            stop_reason = self.thread().GetStopReason()
            if stop_reason == lldb.eStopReasonWatchpoint:
                self.assertFalse(watchpoint_hit, "Watchpoint already hit.")
                expected_stop_desc = "watchpoint %d" % wp_id
                actual_stop_desc = self.thread().stop_description
                self.assertEqual(
                    actual_stop_desc, expected_stop_desc, "Watchpoint ID didn't match."
                )
                watchpoint_hit = True
            else:
                self.assertStopReason(
                    stop_reason, lldb.eStopReasonPlanComplete, STOPPED_DUE_TO_STEP_IN
                )
        self.assertTrue(watchpoint_hit, "Watchpoint never hit.")



"""
Test example snippets from the lldb 'help expression' output.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Radar9673644TestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.main_source = "main.c"
        self.line = line_number(self.main_source, "// Set breakpoint here.")

    def test_expr_commands(self):
        """The following expression commands should just work."""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, self.main_source, self.line, num_expected_locations=1, loc_exact=True
        )

        self.runCmd("run", RUN_SUCCEEDED)

        # rdar://problem/9673664 lldb expression evaluation problem

        self.expect('expr char str[] = "foo"; str[0]', substrs=["'f'"])
        # runCmd: expr char c[] = "foo"; c[0]
        # output: (char) $0 = 'f'



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil

import re

@skipIf(macos_version=[">=", "15.4"], asan=True)
class LibCxxInternalsRecognizerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @add_test_categories(["libc++"])
    @skipIf(compiler="clang", compiler_version=["<", "21.0"])
    def test_frame_recognizer(self):
        """Test that implementation details of libc++ are hidden"""
        self.build()
        (target, process, thread, bkpt) = lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )

        expected_parents = {
            "sort_less(int, int)": ["::sort", "test_algorithms"],
            # `std::ranges::sort` is implemented as an object of types `__sort`.
            # We never hide the frame of the entry-point into the standard library, even
            # if the name starts with `__` which usually indicates an internal function.
            "ranges_sort_less(int, int)": [
                re.compile(r"ranges::__sort::(__fn::)?operator\(\)"),
                "test_algorithms",
            ],
            # `ranges::views::transform` internally uses `std::invoke`, and that
            # call also shows up in the stack trace
            "view_transform(int)": [
                "::invoke",
                "ranges::transform_view",
                "test_algorithms",
            ],
            # Various types of `invoke` calls
            "consume_number(int)": ["::invoke", "test_invoke"],
            "invoke_add(int, int)": ["::invoke", "test_invoke"],
            "Callable::member_function(int) const": ["::invoke", "test_invoke"],
            "Callable::operator()(int) const": ["::invoke", "test_invoke"],
            # Containers
            "MyKey::operator<(MyKey const&) const": [
                "::operator()",
                "less",
                "::emplace",
                "test_containers",
            ],
        }
        stop_set = set()
        while process.GetState() != lldb.eStateExited:
            fn = thread.GetFrameAtIndex(0).GetFunctionName()
            stop_set.add(fn)
            self.assertIn(fn, expected_parents.keys())
            frame_id = 1
            for expected_parent in expected_parents[fn]:
                # Skip all hidden frames
                while (
                    frame_id < thread.GetNumFrames()
                    and thread.GetFrameAtIndex(frame_id).IsHidden()
                ):
                    frame_id = frame_id + 1
                # Expect the correct parent frame
                func_name = thread.GetFrameAtIndex(frame_id).GetFunctionName()
                if isinstance(expected_parent, re.Pattern):
                    self.assertTrue(
                        expected_parent.search(func_name) is not None,
                        f"'{expected_parent}' not found in '{func_name}'"
                    )
                else:
                    self.assertIn(expected_parent, func_name)
                frame_id = frame_id + 1
            process.Continue()

        # Make sure that we actually verified all intended scenarios
        self.assertEqual(len(stop_set), len(expected_parents))



"""
Use lldb Python SBWatchpoint API to set the ignore count.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class WatchpointIgnoreCountTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Our simple source filename.
        self.source = "main.c"
        # Find the line number to break inside main().
        self.line = line_number(self.source, "// Set break point at this line.")

    # on arm64 targets, lldb has incorrect hit-count / ignore-counts
    # for watchpoints when they are hit with multiple threads at
    # the same time.  Tracked as llvm.org/pr49433
    # or rdar://93863107 inside Apple.
    def affected_by_radar_93863107(self):
        return (
            self.getArchitecture() in ["arm64", "arm64e"]
        ) and self.platformIsDarwin()

    # Read-write watchpoints not supported on SystemZ
    @expectedFailureAll(archs=["s390x"])
    def test_set_watch_ignore_count(self):
        """Test SBWatchpoint.SetIgnoreCount() API."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Create a breakpoint on main.c in order to set our watchpoint later.
        breakpoint = target.BreakpointCreateByLocation(self.source, self.line)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        # We should be stopped due to the breakpoint.  Get frame #0.
        process = target.GetProcess()
        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        frame0 = thread.GetFrameAtIndex(0)

        # Watch 'global' for read and write.
        value = frame0.FindValue("global", lldb.eValueTypeVariableGlobal)
        error = lldb.SBError()
        watchpoint = value.Watch(True, True, True, error)
        self.assertTrue(
            value and watchpoint, "Successfully found the variable and set a watchpoint"
        )
        self.DebugSBValue(value)

        # Hide stdout if not running with '-t' option.
        if not self.TraceOn():
            self.HideStdout()

        # There should be only 1 watchpoint location under the target.
        self.assertEqual(target.GetNumWatchpoints(), 1)
        watchpoint = target.GetWatchpointAtIndex(0)
        self.assertTrue(watchpoint.IsEnabled())
        self.assertEqual(watchpoint.GetIgnoreCount(), 0)
        watch_id = watchpoint.GetID()
        self.assertNotEqual(watch_id, 0)
        print(watchpoint)

        # Now immediately set the ignore count to 2.  When we continue, expect the
        # inferior to run to its completion without stopping due to watchpoint.
        watchpoint.SetIgnoreCount(2)
        print(watchpoint)
        process.Continue()

        # At this point, the inferior process should have exited.
        self.assertState(process.GetState(), lldb.eStateExited, PROCESS_EXITED)

        # Verify some vital statistics.
        self.assertTrue(watchpoint)
        self.assertEqual(watchpoint.GetWatchSize(), 4)
        if not self.affected_by_radar_93863107():
            self.assertEqual(watchpoint.GetHitCount(), 2)
        print(watchpoint)



"""
Test top-level expressions.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TopLevelExpressionsTestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.c.
        self.line = line_number("main.cpp", "// Set breakpoint here")
        self.dummy_line = line_number("dummy.cpp", "// Set breakpoint here")

        # Disable confirmation prompt to avoid infinite wait
        self.runCmd("settings set auto-confirm true")
        self.addTearDownHook(lambda: self.runCmd("settings clear auto-confirm"))

    def build_and_run(self):
        """Test top-level expressions."""
        self.build()

        self.runCmd("file " + self.getBuildArtifact("a.out"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self, "main.cpp", self.line, num_expected_locations=1, loc_exact=False
        )

        self.runCmd("run", RUN_SUCCEEDED)

    def run_dummy(self):
        self.runCmd("file " + self.getBuildArtifact("dummy"), CURRENT_EXECUTABLE_SET)

        lldbutil.run_break_set_by_file_and_line(
            self,
            "dummy.cpp",
            self.dummy_line,
            num_expected_locations=1,
            loc_exact=False,
        )

        self.runCmd("run", RUN_SUCCEEDED)

    @add_test_categories(["pyapi"])
    @skipIf(debug_info="gmodules")  # not relevant
    @skipIf(oslist=["windows"])  # Error in record layout on Windows
    def test_top_level_expressions(self):
        self.build_and_run()

        resultFromCode = (
            self.frame().EvaluateExpression("doTest()").GetValueAsUnsigned()
        )

        self.runCmd("kill")

        self.run_dummy()

        codeFile = open("test.cpp", "r")

        expressions = []
        current_expression = ""

        for line in codeFile:
            if line.startswith("// --"):
                expressions.append(current_expression)
                current_expression = ""
            else:
                current_expression += line

        options = lldb.SBExpressionOptions()
        options.SetLanguage(lldb.eLanguageTypeC_plus_plus)
        options.SetTopLevel(True)

        for expression in expressions:
            self.frame().EvaluateExpression(expression, options)

        resultFromTopLevel = self.frame().EvaluateExpression("doTest()")

        self.assertTrue(resultFromTopLevel.IsValid())
        self.assertEqual(resultFromCode, resultFromTopLevel.GetValueAsUnsigned())

        # Make sure the command line version works as well:
        self.runCmd("expr --top-level -- int TopLevelFunction() { return 101; }")
        resultFromTopLevel = self.frame().EvaluateExpression("TopLevelFunction()")
        self.assertTrue(resultFromTopLevel.IsValid())
        self.assertEqual(
            101, resultFromTopLevel.GetValueAsUnsigned(), "Command line version works."
        )

    def test_top_level_expression_without_target(self):
        self.expect(
            "expr --top-level -- void func() {}",
            error=True,
            substrs=["Top-level code needs to be inserted into a runnable target"],
        )

        # FIXME: This doesn't actually generate any code, so LLDB should probably
        # allow these expressions.
        self.expect(
            "expr --top-level -- template<typename T> struct StructT { T m; };",
            error=True,
            substrs=["Top-level code needs to be inserted into a runnable target"],
        )
        self.expect(
            "expr --top-level -- struct Struct { int i; };",
            error=True,
            substrs=["Top-level code needs to be inserted into a runnable target"],
        )



"""
Tests GCC's complex integer types.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @no_debug_info_test
    @skipIfWindows
    def test(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        long_size_eq_int = self.frame().EvaluateExpression(
            "sizeof(long) == sizeof(int)"
        )

        # FIXME: LLDB treats all complex ints as unsigned, so the value is wrong.
        self.expect_expr(
            "complex_int",
            result_type="_Complex int",
            result_value="4294967295 + 4294967294i",
        )
        self.expect_expr(
            "complex_unsigned", result_type="_Complex int", result_value="1 + 2i"
        )

        # FIXME: We get the type wrong if long has the same size as int.
        if long_size_eq_int.GetValue() == "true":
            self.expect_expr("complex_long", result_type="_Complex int")
            self.expect_expr(
                "complex_unsigned_long",
                result_type="_Complex int",
                result_value="1 + 2i",
            )
        else:
            self.expect_expr("complex_long", result_type="_Complex long")
            self.expect_expr(
                "complex_unsigned_long",
                result_type="_Complex long",
                result_value="1 + 2i",
            )

    @no_debug_info_test
    @skipIfWindows
    def test_long_long(self):
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.c")
        )

        # FIXME: We get the type wrong if long has the same size as long long.
        # FIXME: LLDB treats all complex ints as unsigned, so the value is wrong.
        long_size_eq_long_long = self.frame().EvaluateExpression(
            "sizeof(long) == sizeof(long long)"
        )
        if long_size_eq_long_long.GetValue() == "true":
            self.expect_expr("complex_long_long", result_type="_Complex long")
            self.expect_expr(
                "complex_unsigned_long_long",
                result_type="_Complex long",
                result_value="1 + 2i",
            )
        else:
            self.expect_expr("complex_long_long", result_type="_Complex long long")
            self.expect_expr(
                "complex_unsigned_long_long",
                result_type="_Complex long long",
                result_value="1 + 2i",
            )



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCoroutineHandle(TestBase):
    SHARED_BUILD_TESTCASE = False

    def do_test(self):
        """Test std::coroutine_handle is displayed correctly."""
        is_clang = self.expectedCompiler(["clang"])

        # Clang <= 20 used to also name the resume/destroy functions
        # as `my_generator_func`.
        # Never versions of clang name the clones as `.resume`/`.destroy`.
        test_generator_func_ptr_re = re.compile(
            r"^\(a.out`my_generator_func\(\)( \(\..*\))? at main.cpp:[0-9]*\)$"
        )

        # Run until the initial suspension point
        lldbutil.run_to_source_breakpoint(
            self, "// Break at initial_suspend", lldb.SBFileSpec("main.cpp", False)
        )

        if self.frame().FindVariable("is_supported").GetValueAsUnsigned(1) == 0:
            self.skipTest("c++ library not supported")

        # Check that we show the correct function pointers and the `promise`.
        self.expect_expr(
            "gen.hdl",
            result_summary=re.compile("^coro frame = 0x[0-9a-f]*$"),
            result_children=[
                ValueCheck(name="resume", summary=test_generator_func_ptr_re),
                ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                ValueCheck(
                    name="promise",
                    children=[
                        ValueCheck(name="current_value", value="-1"),
                    ],
                ),
                # We don not check any members inside the `coro_frame`,
                # as its contents are highly compiler-specific.
                ValueCheck(name="coro_frame"),
            ],
        )

        # For a type-erased `coroutine_handle<>`, we can still devirtualize
        # the promise call and display the correctly typed promise. This
        # currently only works in clang, because gcc is not adding the
        # artificial `__promise` variable to the destroy function.
        if is_clang:
            self.expect_expr(
                "type_erased_hdl",
                result_summary=re.compile("^coro frame = 0x[0-9a-f]*$"),
                result_children=[
                    ValueCheck(name="resume", summary=test_generator_func_ptr_re),
                    ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                    ValueCheck(
                        name="promise",
                        children=[
                            ValueCheck(name="current_value", value="-1"),
                        ],
                    ),
                    ValueCheck(name="coro_frame"),
                ],
            )

        # For an incorrectly typed `coroutine_handle`, we use the user-supplied
        # incorrect type instead of inferring the correct type. Strictly speaking,
        # incorrectly typed coroutine handles are undefined behavior. However,
        # it provides probably a better debugging experience if we display the
        # promise as seen by the program instead of fixing this bug based on
        # the available debug info.
        self.expect_expr(
            "incorrectly_typed_hdl",
            result_summary=re.compile("^coro frame = 0x[0-9a-f]*$"),
            result_children=[
                ValueCheck(name="resume", summary=test_generator_func_ptr_re),
                ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                ValueCheck(name="promise", dereference=ValueCheck(value="-1")),
                ValueCheck(name="coro_frame"),
            ],
        )

        process = self.process()

        # Break at a coroutine body
        lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "int_generator my_generator_func",
            lldb.SBFileSpec("main.cpp", False),
        )

        # Expect artificial variables to be displayed
        self.expect("frame variable", substrs=["__promise", "__coro_frame"])

        # Run until after the `co_yield`
        lldbutil.continue_to_source_breakpoint(
            self, process, "// Break after co_yield", lldb.SBFileSpec("main.cpp", False)
        )
        # We correctly show the updated value inside `promise.current_value`.
        self.expect_expr(
            "gen.hdl",
            result_children=[
                ValueCheck(name="resume", summary=test_generator_func_ptr_re),
                ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                ValueCheck(
                    name="promise",
                    children=[
                        ValueCheck(name="current_value", value="42"),
                    ],
                ),
                ValueCheck(name="coro_frame"),
            ],
        )

        # Run until the `final_suspend`
        lldbutil.continue_to_source_breakpoint(
            self,
            process,
            "// Break at final_suspend",
            lldb.SBFileSpec("main.cpp", False),
        )
        # At the final suspension point, `resume` is set to a nullptr.
        # Check that we still show the remaining data correctly.
        self.expect_expr(
            "gen.hdl",
            result_children=[
                ValueCheck(name="resume", value=re.compile("^0x0+$")),
                ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                ValueCheck(
                    name="promise",
                    children=[
                        ValueCheck(name="current_value", value="42"),
                    ],
                ),
                ValueCheck(name="coro_frame"),
            ],
        )
        if is_clang:
            # Devirtualization still works, also at the final suspension point, despite
            # the `resume` pointer being reset to a nullptr
            self.expect_expr(
                "type_erased_hdl",
                result_summary=re.compile("^coro frame = 0x[0-9a-f]*$"),
                result_children=[
                    ValueCheck(name="resume", value=re.compile("^0x0+$")),
                    ValueCheck(name="destroy", summary=test_generator_func_ptr_re),
                    ValueCheck(
                        name="promise",
                        children=[
                            ValueCheck(name="current_value", value="42"),
                        ],
                    ),
                    ValueCheck(name="coro_frame"),
                ],
            )

    @add_test_categories(["libstdcxx"])
    def test_libstdcpp(self):
        self.build(dictionary={"USE_LIBSTDCPP": 1})
        self.do_test()

    @add_test_categories(["libc++"])
    @skipIf(compiler="clang", compiler_version=["<", "15.0"])
    def test_libcpp(self):
        self.build(dictionary={"USE_LIBCPP": 1})
        self.do_test()



#!/usr/bin/env python

import subprocess
import sys

args = sys.argv

expected_exit_code = args[1]

args = args[2:]
print("Running " + (" ".join(args)))
real_exit_code = subprocess.call(args)

if str(real_exit_code) != expected_exit_code:
    print("Got exit code %d but expected %s" % (real_exit_code, expected_exit_code))
    exit(1)



import os
import struct
import subprocess

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class FreeBSDKernelLiveCoreTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_mem(self):
        kernel_exec = "/boot/kernel/kernel"
        mem_device = "/dev/mem"

        if not os.access(kernel_exec, os.R_OK):
            self.skipTest("Kernel @ %s is not readable" % (kernel_exec,))
        if not os.access(mem_device, os.R_OK):
            self.skipTest("Memory @ %s is not readable" % (mem_device,))

        target = self.dbg.CreateTarget(kernel_exec)
        process = target.LoadCore(mem_device)
        hz_value = int(subprocess.check_output(["sysctl", "-n", "kern.hz"]))

        self.assertTrue(process, PROCESS_IS_VALID)
        self.assertEqual(process.GetNumThreads(), 1)
        self.assertEqual(process.GetProcessID(), 0)

        # test memory reading
        self.expect("expr -- *(int *) &hz", substrs=["(int) $0 = %d" % hz_value])

        main_mod = target.GetModuleAtIndex(0)
        hz_addr = main_mod.FindSymbols("hz")[0].symbol.addr.GetLoadAddress(target)
        error = lldb.SBError()
        self.assertEqual(
            process.ReadMemory(hz_addr, 4, error), struct.pack("<I", hz_value)
        )

        self.dbg.DeleteTarget(target)



from dap_server import Source
import shutil
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import lldbdap_testcase
import os
import lldb


class TestDAP_InstructionBreakpointTestCase(lldbdap_testcase.DAPTestCaseBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        lldbdap_testcase.DAPTestCaseBase.setUp(self)

        self.main_basename = "main-copy.cpp"
        self.main_path = os.path.realpath(self.getBuildArtifact(self.main_basename))

    @skipIfWindows
    def test_instruction_breakpoint(self):
        self.build()
        self.instruction_breakpoint_test()

    def instruction_breakpoint_test(self):
        """Sample test to ensure SBFrame::Disassemble produces SOME output"""
        # Create a target by the debugger.
        target = self.createTestTarget()

        main_line = line_number("main.cpp", "breakpoint 1")

        program = self.getBuildArtifact("a.out")
        self.build_and_launch(program)

        # Set source breakpoint 1
        response = self.dap_server.request_setBreakpoints(
            Source.build(path=self.main_path), [main_line]
        )
        breakpoints = response["body"]["breakpoints"]
        self.assertEqual(len(breakpoints), 1)
        breakpoint = breakpoints[0]
        self.assertEqual(
            breakpoint["line"], main_line, "incorrect breakpoint source line"
        )
        self.assertTrue(breakpoint["verified"], "breakpoint is not verified")
        self.assertEqual(
            self.main_basename, breakpoint["source"]["name"], "incorrect source name"
        )
        self.assertEqual(
            self.main_path, breakpoint["source"]["path"], "incorrect source file path"
        )
        other_breakpoint_id = breakpoint["id"]

        # Continue and then verifiy the breakpoint
        self.dap_server.request_continue()
        self.verify_breakpoint_hit([other_breakpoint_id])

        # now we check the stack trace making sure that we got mapped source paths
        frames = self.dap_server.request_stackTrace()["body"]["stackFrames"]
        intstructionPointerReference = []
        setIntstructionBreakpoints = []
        intstructionPointerReference.append(frames[0]["instructionPointerReference"])
        self.assertEqual(
            frames[0]["source"]["name"], self.main_basename, "incorrect source name"
        )
        self.assertEqual(
            frames[0]["source"]["path"], self.main_path, "incorrect source file path"
        )

        # Check disassembly view
        disassembled_instructions, instruction = self.disassemble(frameIndex=0)
        self.assertEqual(
            instruction["address"],
            intstructionPointerReference[0],
            "current breakpoint reference is not in the disaasembly view",
        )

        # Get next instruction address to set instruction breakpoint
        instruction_addr_list = list(disassembled_instructions.keys())
        index = instruction_addr_list.index(intstructionPointerReference[0])
        if len(instruction_addr_list) >= (index + 1):
            next_inst_addr = instruction_addr_list[index + 1]
            if len(next_inst_addr) > 2:
                setIntstructionBreakpoints.append(next_inst_addr)
                instruction_breakpoint_response = (
                    self.dap_server.request_setInstructionBreakpoints(
                        setIntstructionBreakpoints
                    )
                )
                inst_breakpoints = instruction_breakpoint_response["body"][
                    "breakpoints"
                ]
                self.assertEqual(
                    inst_breakpoints[0]["instructionReference"],
                    next_inst_addr,
                    "Instruction breakpoint has not been resolved or failed to relocate the instruction breakpoint",
                )
                self.dap_server.request_continue()
                self.verify_breakpoint_hit([inst_breakpoints[0]["id"]])



import lldb


class fooSynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj
        self.int_type = valobj.GetType().GetBasicType(lldb.eBasicTypeInt)

    def num_children(self):
        return 3

    def get_child_at_index(self, index):
        if index == 0:
            child = self.valobj.GetChildMemberWithName("a")
        if index == 1:
            child = self.valobj.CreateChildAtOffset("fake_a", 1, self.int_type)
        if index == 2:
            child = self.valobj.GetChildMemberWithName("r")
        return child

    def get_child_index(self, name):
        if name == "a":
            return 0
        if name == "fake_a":
            return 1
        return 2

    def update(self):
        return True


class wrapfooSynthProvider:
    def __init__(self, valobj, dict):
        self.valobj = valobj

    def num_children(self):
        return 1

    def get_child_at_index(self, index):
        if index == 0:
            return self.valobj.GetChildMemberWithName("ptr")
        if index == 1:
            return self.valobj.GetChildMemberWithName("ptr").Dereference()
        return None

    def get_child_index(self, name):
        if name == "ptr":
            return 0
        if name == "$$dereference$$":
            return 1
        return -1

    def update(self):
        return True



"""Test the SBError APIs."""

from lldbsuite.test.lldbtest import *


class TestSBError(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_generic_error(self):
        error = lldb.SBError()
        error.SetErrorToGenericError()
        self.assertEqual(error.GetType(), lldb.eErrorTypeGeneric)



# -*- Python -*-

import os

import lit.formats

# name: The name of this test suite.
config.name = "flang-rt-Unit"

# suffixes: A list of file extensions to treat as test files.
config.suffixes = []

# test_source_root: The root path where unit test binaries are located.
config.test_source_root = os.path.join(config.flangrt_binary_dir, "unittests")

# test_exec_root: The root path where tests should be run.
# lit writes a '.lit_test_times.txt' file into this directory.
config.test_exec_root = config.flang_rt_binary_test_dir

# testFormat: The test format to use to interpret tests.
config.test_format = lit.formats.GoogleTest(config.llvm_build_mode, "Tests")



# coding=utf8
"""
Test that the C++11 support for char16_t and char32_t works correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class Char1632TestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break for main.cpp.
        self.source = "main.cpp"
        self.lines = [
            line_number(self.source, "// breakpoint1"),
            line_number(self.source, "// breakpoint2"),
        ]

    @expectedFailureAll(
        compiler="icc",
        bugnumber="ICC (13.1) does not emit the DW_TAG_base_type for char16_t and char32_t.",
    )
    def test(self):
        """Test that the C++11 support for char16_t and char32_t works correctly."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set breakpoints
        for line in self.lines:
            lldbutil.run_break_set_by_file_and_line(self, "main.cpp", line)

        # Now launch the process, and do not stop at entry point and stop at
        # breakpoint1
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        if not process:
            self.fail("SBTarget.Launch() failed")

        if self.TraceOn():
            self.runCmd("frame variable")

        # Check that we correctly report the const types
        self.expect(
            "frame variable cs16 cs32",
            substrs=[
                "(const char16_t *) cs16 = ",
                'u"hello world ྒྙྐ"',
                "(const char32_t *) cs32 = ",
                'U"hello world ྒྙྐ"',
            ],
        )

        # Check that we correctly report the non-const types
        self.expect(
            "frame variable s16 s32",
            substrs=[
                "(char16_t *) s16 = ",
                'u"ﺸﺵۻ"',
                "(char32_t *) s32 = ",
                'U"ЕЙРГЖО"',
            ],
        )

        # Check that we correctly report the array types
        self.expect(
            "frame variable as16 as32",
            patterns=[
                r"\(char16_t\[[0-9]+\]\) as16 = ",
                r"\(char32_t\[[0-9]+\]\) as32 = ",
            ],
            substrs=['u"ﺸﺵۻ"', 'U"ЕЙРГЖО"'],
        )

        self.runCmd("next")  # step to after the string is nullified

        # check that we don't crash on NULL
        self.expect("frame variable s32", substrs=["(char32_t *) s32 = 0x00000000"])

        # continue and hit breakpoint2
        self.runCmd("continue")

        # check that the new strings show
        self.expect(
            "frame variable s16 s32",
            substrs=[
                "(char16_t *) s16 = 0x",
                '"色ハ匂ヘト散リヌルヲ"',
                "(char32_t *) s32 = ",
                '"෴"',
            ],
        )

        # check the same as above for arrays
        self.expect(
            "frame variable as16 as32",
            patterns=[
                r"\(char16_t\[[0-9]+\]\) as16 = ",
                r"\(char32_t\[[0-9]+\]\) as32 = ",
            ],
            substrs=['"色ハ匂ヘト散リヌルヲ"', '"෴"'],
        )

        # check that zero values are properly handles
        self.expect_expr("cs16_zero", result_summary="U+0000 u'\\0'")
        self.expect_expr("cs32_zero", result_summary="U+0x00000000 U'\\0'")

        # Check that we can run expressions that return charN_t
        self.expect_expr("u'a'", result_type="char16_t", result_summary="U+0061 u'a'")
        self.expect_expr(
            "U'a'", result_type="char32_t", result_summary="U+0x00000061 U'a'"
        )



# coding=utf8
"""
Test that C++ supports char8_t correctly.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class CxxChar8_tTestCase(TestBase):
    @skipIfDarwin  # Chained Fixups
    @skipIf(compiler="clang", compiler_version=["<", "7.0"])
    def test_without_process(self):
        """Test that C++ supports char8_t without a running process."""
        self.build()
        lldbutil.run_to_breakpoint_make_target(self)

        self.expect("target variable a", substrs=["char8_t", "0x61 u8'a'"])
        self.expect("target variable ab", substrs=["const char8_t *", 'u8"你好"'])
        self.expect("target variable abc", substrs=["char8_t[9]", 'u8"你好"'])

        self.expect_expr("a", result_type="char8_t", result_summary="0x61 u8'a'")
        self.expect_expr("ab", result_type="const char8_t *", result_summary='u8"你好"')
        self.expect_expr("abc", result_type="char8_t[9]", result_summary='u8"你好"')

    @skipIf(compiler="clang", compiler_version=["<", "7.0"])
    def test_with_process(self):
        """Test that C++ supports char8_t with a running process."""
        self.build()
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp")
        )

        # As well as with it
        self.expect_expr("a", result_type="char8_t", result_summary="0x61 u8'a'")
        self.expect_expr("ab", result_type="const char8_t *", result_summary='u8"你好"')
        self.expect_expr("abc", result_type="char8_t[9]", result_summary='u8"你好"')



"""
Test that objective-c constant strings are generated correctly by the expression
parser.
"""


import shutil
import subprocess
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestObjCBreakpoints(TestBase):
    SHARED_BUILD_TESTCASE = False

    @add_test_categories(["objc"])
    def test_break(self):
        """Test setting Objective-C specific breakpoints (DWARF in .o files)."""
        self.build()
        self.setTearDownCleanup()
        self.check_objc_breakpoints(False)

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to break inside main().
        self.main_source = "main.m"
        self.line = line_number(self.main_source, "// Set breakpoint here")

    def check_category_breakpoints(self):
        name_bp = self.target.BreakpointCreateByName("myCategoryFunction")
        selector_bp = self.target.BreakpointCreateByName(
            "myCategoryFunction",
            lldb.eFunctionNameTypeSelector,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(
            name_bp.GetNumLocations(),
            selector_bp.GetNumLocations(),
            'Make sure setting a breakpoint by name "myCategoryFunction" sets a breakpoint even though it is in a category',
        )
        for bp_loc in selector_bp:
            function_name = bp_loc.GetAddress().GetSymbol().GetName()
            self.assertIn(
                " myCategoryFunction]",
                function_name,
                'Make sure all function names have " myCategoryFunction]" in their names',
            )

        category_bp = self.target.BreakpointCreateByName(
            "-[MyClass(MyCategory) myCategoryFunction]"
        )
        stripped_bp = self.target.BreakpointCreateByName(
            "-[MyClass myCategoryFunction]"
        )
        stripped2_bp = self.target.BreakpointCreateByName(
            "[MyClass myCategoryFunction]"
        )
        self.assertEqual(
            category_bp.GetNumLocations(),
            1,
            "Make sure we can set a breakpoint using a full objective C function name with the category included (-[MyClass(MyCategory) myCategoryFunction])",
        )
        self.assertEqual(
            stripped_bp.GetNumLocations(),
            1,
            "Make sure we can set a breakpoint using a full objective C function name without the category included (-[MyClass myCategoryFunction])",
        )
        self.assertEqual(
            stripped2_bp.GetNumLocations(),
            1,
            "Make sure we can set a breakpoint using a full objective C function name without the category included ([MyClass myCategoryFunction])",
        )

    def check_objc_breakpoints(self, have_dsym):
        """Test constant string generation amd comparison by the expression parser."""

        # Set debugger into synchronous mode
        self.dbg.SetAsync(False)

        # Create a target by the debugger.
        exe = self.getBuildArtifact("a.out")
        self.target = self.dbg.CreateTarget(exe)
        self.assertTrue(self.target, VALID_TARGET)

        # ----------------------------------------------------------------------
        # Set breakpoints on all selectors whose name is "count". This should
        # catch breakpoints that are both C functions _and_ anything whose
        # selector is "count" because just looking at "count" we can't tell
        # definitively if the name is a selector or a C function
        # ----------------------------------------------------------------------
        name_bp = self.target.BreakpointCreateByName("count")
        selector_bp = self.target.BreakpointCreateByName(
            "count",
            lldb.eFunctionNameTypeSelector,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertGreaterEqual(
            name_bp.GetNumLocations(),
            selector_bp.GetNumLocations(),
            'Make sure we get at least the same amount of breakpoints if not more when setting by name "count"',
        )
        self.assertGreater(
            selector_bp.GetNumLocations(),
            50,
            'Make sure we find a lot of "count" selectors',
        )  # There are 93 on the latest MacOSX
        for bp_loc in selector_bp:
            function_name = bp_loc.GetAddress().GetSymbol().GetName()
            self.assertIn(
                " count]",
                function_name,
                'Make sure all function names have " count]" in their names',
            )

        # ----------------------------------------------------------------------
        # Set breakpoints on all selectors whose name is "isEqual:". This should
        # catch breakpoints that are only ObjC selectors because no C function
        # can end with a :
        # ----------------------------------------------------------------------
        name_bp = self.target.BreakpointCreateByName("isEqual:")
        selector_bp = self.target.BreakpointCreateByName(
            "isEqual:",
            lldb.eFunctionNameTypeSelector,
            lldb.SBFileSpecList(),
            lldb.SBFileSpecList(),
        )
        self.assertEqual(
            name_bp.GetNumLocations(),
            selector_bp.GetNumLocations(),
            'Make sure setting a breakpoint by name "isEqual:" only sets selector breakpoints',
        )
        for bp_loc in selector_bp:
            function_name = bp_loc.GetAddress().GetSymbol().GetName()
            self.assertIn(
                " isEqual:]",
                function_name,
                'Make sure all function names have " isEqual:]" in their names',
            )

        self.check_category_breakpoints()

        if have_dsym:
            shutil.rmtree(exe + ".dSYM")
        self.assertEqual(
            subprocess.call(["/usr/bin/strip", "-Sx", exe]),
            0,
            "stripping dylib succeeded",
        )

        # Check breakpoints again, this time using the symbol table only
        self.check_category_breakpoints()



"""
Test that the arm64 ADRP + ADD pc-relative addressing pair is symbolicated.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestAArch64AdrpAdd(TestBase):
    @no_debug_info_test
    @skipIfLLVMTargetMissing("AArch64")
    def test_arm64(self):
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "a.out-arm64.yaml")
        obj_path = self.getBuildArtifact("a.out-arm64")
        self.yaml2obj(yaml_path, obj_path)

        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        mains = target.FindFunctions("main")
        for f in mains.symbols:
            binaryname = f.GetStartAddress().GetModule().GetFileSpec().GetFilename()
            if binaryname == "a.out-arm64":
                self.disassemble_check_for_hi_and_foo(target, f, binaryname)

    @no_debug_info_test
    @skipIfLLVMTargetMissing("AArch64")
    def test_arm64_32(self):
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "a.out-arm64_32.yaml")
        obj_path = self.getBuildArtifact("a.out-arm64_32")
        self.yaml2obj(yaml_path, obj_path)

        target = self.dbg.CreateTarget(obj_path)
        self.assertTrue(target, VALID_TARGET)

        mains = target.FindFunctions("main")
        for f in mains.symbols:
            binaryname = f.GetStartAddress().GetModule().GetFileSpec().GetFilename()
            if binaryname == "a.out-arm64_32":
                self.disassemble_check_for_hi_and_foo(target, f, binaryname)

    def disassemble_check_for_hi_and_foo(self, target, func, binaryname):
        insns = func.GetInstructions(target)
        found_hi_string = False
        found_foo = False

        # The binary has an ADRP + ADD instruction pair which load
        # the pc-relative address of a c-string, and loads the address
        # of a function into a function pointer.  lldb should show
        # that c-string and the name of that function in the disassembly
        # comment field.
        for i in insns:
            if "HI" in i.GetComment(target):
                found_hi_string = True
            if "foo" in i.GetComment(target):
                found_foo = True
        if not found_hi_string or not found_foo:
            print(
                'Did not find "HI" string or "foo" in disassembly symbolication in %s'
                % binaryname
            )
            if self.TraceOn():
                strm = lldb.SBStream()
                insns.GetDescription(strm)
                print('Disassembly of main(), looking for "HI" and "foo" in comments:')
                print(strm.GetData())
        self.assertTrue(found_hi_string)
        self.assertTrue(found_foo)



"""
Fuzz tests an object after the default construction to make sure it does not crash lldb.
"""

import lldb


def fuzz_obj(obj):
    obj.AddEvent(lldb.SBEvent())
    obj.StartListeningForEvents(lldb.SBBroadcaster(), 0xFFFFFFFF)
    obj.StopListeningForEvents(lldb.SBBroadcaster(), 0xFFFFFFFF)
    event = lldb.SBEvent()
    broadcaster = lldb.SBBroadcaster()
    obj.WaitForEvent(5, event)
    obj.WaitForEventForBroadcaster(5, broadcaster, event)
    obj.WaitForEventForBroadcasterWithType(5, broadcaster, 0xFFFFFFFF, event)
    obj.PeekAtNextEvent(event)
    obj.PeekAtNextEventForBroadcaster(broadcaster, event)
    obj.PeekAtNextEventForBroadcasterWithType(broadcaster, 0xFFFFFFFF, event)
    obj.GetNextEvent(event)
    obj.GetNextEventForBroadcaster(broadcaster, event)
    obj.GetNextEventForBroadcasterWithType(broadcaster, 0xFFFFFFFF, event)
    obj.HandleBroadcastEvent(event)



import lldb
import os
from lldbsuite.test.lldbtest import *
from lldbsuite.test.decorators import *


@skipIfRemote
class TestQemuAPI(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test_file_api(self):
        qemu = lldb.SBPlatform("qemu-user")
        host = lldb.SBPlatform.GetHostPlatform()

        target = self.getBuildArtifact("target.c")
        main_c = lldb.SBFileSpec(self.getSourcePath("main.c"))

        self.assertSuccess(qemu.Put(main_c, lldb.SBFileSpec(target)))
        self.assertTrue(os.path.exists(target))
        self.assertEqual(
            qemu.GetFilePermissions(target), host.GetFilePermissions(target)
        )

        self.assertSuccess(qemu.MakeDirectory(self.getBuildArtifact("target_dir")))
        self.assertTrue(os.path.isdir(self.getBuildArtifact("target_dir")))

    def test_process_api(self):
        self.dbg.SetCurrentPlatform("qemu-user")
        self.expect(
            "platform process list",
            substrs=['no processes were found on the "qemu-user" platform'],
            error=True,
        )



"""
Check that missing module source files are correctly handled by LLDB.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os
import shutil


class TestCase(TestBase):
    # We only emulate a fake libc++ in this test and don't use the real libc++,
    # but we still add the libc++ category so that this test is only run in
    # test configurations where libc++ is actually supposed to be tested.
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    @skipIfRemote
    def test(self):
        # The path to our temporary target root that contains the temporary
        # module sources.
        target_sysroot = self.getBuildArtifact("root")

        # Copy the sources to the root.
        shutil.copytree(self.getSourcePath("root"), target_sysroot)
        # Build the binary with the copied sources.
        self.build()
        # Delete the copied sources so that they are now unavailable.
        shutil.rmtree(target_sysroot)

        # Set the sysroot where our dummy libc++ used to exist. Just to make
        # sure we don't find some existing headers on the system that could
        # XPASS this test.
        self.runCmd("platform select --sysroot '" + target_sysroot + "' host")

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        # Import the std C++ module and run an expression.
        # As we deleted the sources, LLDB should refuse the load the module
        # and just print the normal error we get from the expression.
        self.runCmd("settings set target.import-std-module true")
        self.expect(
            "expr v.unknown_identifier",
            error=True,
            substrs=["no member named 'unknown_identifier'"],
        )
        # Check that there is no confusing error about failing to build the
        # module.
        self.expect(
            "expr v.unknown_identifier",
            error=True,
            matching=False,
            substrs=["could not build module 'std'"],
        )

        # Test the fallback mode. It should also just print the normal
        # error but not mention a failed module build.
        self.runCmd("settings set target.import-std-module fallback")

        self.expect(
            "expr v.unknown_identifier",
            error=True,
            substrs=["no member named 'unknown_identifier'"],
        )
        self.expect(
            "expr v.unknown_identifier",
            error=True,
            matching=False,
            substrs=["could not build module 'std'"],
        )



from lldbsuite.test import lldbinline
from lldbsuite.test import decorators

lldbinline.MakeInlineTest(
    __file__,
    globals(),
    [
        decorators.expectedFailureAll(
            bugnumber="https://github.com/llvm/llvm-project/issues/149477"
        )
    ],
)



"""
Test lldb data formatter subsystem.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestDataFormatterStdQueue(TestBase):
    SHARED_BUILD_TESTCASE = False

    def setUp(self):
        TestBase.setUp(self)
        self.namespace = "std"

    def check_variable(self, name):
        var = self.frame().FindVariable(name)
        self.assertTrue(var.IsValid())

        queue = self.namespace + "::queue"
        self.assertIn(queue, var.GetDisplayTypeName())
        self.assertEqual(var.GetNumChildren(), 5)
        for i in range(5):
            ch = var.GetChildAtIndex(i)
            self.assertTrue(ch.IsValid())
            self.assertEqual(ch.GetValueAsSigned(), i + 1)

    @expectedFailureAll(
        bugnumber="llvm.org/pr36109", debug_info="gmodules", triple=".*-android"
    )
    @add_test_categories(["libc++"])
    def test_libcxx(self):
        """Test that std::queue is displayed correctly"""
        self.build(dictionary={"USE_LIBCPP": 1})
        lldbutil.run_to_source_breakpoint(
            self, "// break here", lldb.SBFileSpec("main.cpp", False)
        )

        self.check_variable("q1")
        self.check_variable("q2")



"""
Test basic std::pair functionality.
"""

from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @add_test_categories(["libc++"])
    @skipIf(compiler=no_match("clang"))
    # FIXME: This regressed in 69d5a6662115499198ebfa07a081e98a6ce4b915
    # but needs further investigation for what underlying Clang/LLDB bug can't
    # handle that code change.
    @skip
    def test(self):
        self.build()

        lldbutil.run_to_source_breakpoint(
            self, "// Set break point at this line.", lldb.SBFileSpec("main.cpp")
        )

        self.runCmd("settings set target.import-std-module true")

        self.expect_expr("pair_int.first", result_type="int", result_value="1234")
        self.expect_expr("pair_int.second", result_type="int", result_value="5678")
        self.expect_expr(
            "pair_int",
            result_type="std::pair<int, int>",
            result_children=[
                ValueCheck(name="first", value="1234"),
                ValueCheck(name="second", value="5678"),
            ],
        )
        self.expect_expr(
            "std::pair<long, long> lp; lp.first = 3333; lp.second = 2344; lp",
            result_type="std::pair<long, long>",
            result_children=[
                ValueCheck(name="first", value="3333"),
                ValueCheck(name="second", value="2344"),
            ],
        )



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    def test(self):
        self.build()
        self.createTestTarget()

        # Empty structs are not allowed in C, but Clang/GCC allow them and
        # give them a size of 0.
        self.expect_expr("sizeof(Empty) == sizeof_empty", result_value="true")
        self.expect_expr(
            "sizeof(EmptyMember) == sizeof_empty_member", result_value="true"
        )
        self.expect_expr("sizeof(SingleMember) == sizeof_single", result_value="true")
        self.expect_expr("sizeof(PaddingMember) == sizeof_padding", result_value="true")



"""
This test verifies the correct handling of the situation when a thread exits
while another thread triggers the termination (exit) of the entire process.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
import lldbsuite.test.lldbutil as lldbutil


class ConcurrentThreadExitTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    @skipIf(oslist=no_match(["linux"]))
    def test(self):
        self.build()
        exe = self.getBuildArtifact("a.out")
        self.runCmd("file " + exe, CURRENT_EXECUTABLE_SET)
        self.expect("run", substrs=["exited with status = 47"])



"""Test that types defined in shared libraries work correctly."""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestRealDefinition(TestBase):
    def test_frame_var_after_stop_at_interface(self):
        """Test that we can find the implementation for an objective C type"""
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        self.build()

        lldbutil.run_to_source_breakpoint(
            self,
            "// Set breakpoint where Bar is an interface",
            lldb.SBFileSpec("Foo.m", False),
        )

        # Break inside the foo function which takes a bar_ptr argument.
        self.expect('breakpoint set -p "// Set breakpoint in main"')
        self.runCmd("continue", RUN_SUCCEEDED)

        # Run at stop at main
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # This should display correctly.
        self.expect(
            "frame variable foo->_bar->_hidden_ivar",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["foo->_bar->_hidden_ivar = 0x"],
        )

    def test_frame_var_after_stop_at_implementation(self):
        """Test that we can find the implementation for an objective C type"""
        if self.getArchitecture() == "i386":
            self.skipTest("requires modern objc runtime")
        self.build()

        lldbutil.run_to_source_breakpoint(
            self,
            "// Set breakpoint where Bar is an implementation",
            lldb.SBFileSpec("Bar.m", False),
        )

        self.expect('breakpoint set -p "// Set breakpoint in main"')
        self.runCmd("continue", RUN_SUCCEEDED)

        # Run at stop at main
        lldbutil.check_breakpoint(self, bpno=1, expected_hit_count=1)

        # This should display correctly.
        self.expect(
            "frame variable foo->_bar->_hidden_ivar",
            VARIABLES_DISPLAYED_CORRECTLY,
            substrs=["foo->_bar->_hidden_ivar = 0x"],
        )



import gdbremote_testcase
import lldbgdbserverutils
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestGdbRemoteKill(gdbremote_testcase.GdbRemoteTestCaseBase):
    def test_attach_commandline_kill_after_initial_stop(self):
        self.build()
        self.set_inferior_startup_attach()
        reg_expr = r"^\$[XW][0-9a-fA-F]+([^#]*)#[0-9A-Fa-f]{2}"
        procs = self.prep_debug_monitor_and_inferior()
        self.test_sequence.add_log_lines(
            [
                "read packet: $k#6b",
                {"direction": "send", "regex": reg_expr},
            ],
            True,
        )

        if self.stub_sends_two_stop_notifications_on_kill:
            # Add an expectation for a second X result for stubs that send two
            # of these.
            self.test_sequence.add_log_lines(
                [
                    {"direction": "send", "regex": reg_expr},
                ],
                True,
            )

        self.expect_gdbremote_sequence()

        # Wait a moment for completed and now-detached inferior process to
        # clear.
        time.sleep(self.DEFAULT_SLEEP)

        if not lldb.remote_platform:
            # Process should be dead now. Reap results.
            poll_result = procs["inferior"].poll()
            self.assertIsNotNone(poll_result)

        # Where possible, verify at the system level that the process is not
        # running.
        self.assertFalse(
            lldbgdbserverutils.process_is_running(procs["inferior"].pid, False)
        )



"""
Test that we embed the swig version into the lldb module
"""


"""
import os
import time
import re
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test import lldbutil
"""
from lldbsuite.test.lldbtest import *


class SwigVersionTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def test(self):
        self.assertTrue(getattr(lldb, "swig_version"))
        self.assertIsInstance(lldb.swig_version, tuple)
        self.assertEqual(len(lldb.swig_version), 3)
        self.assertGreaterEqual(lldb.swig_version[0], 4)
        for v in lldb.swig_version:
            self.assertGreaterEqual(v, 0)



"""
Test that target var can resolve complex DWARF expressions.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class targetCommandTestCase(TestBase):
    @skipIfDarwinEmbedded  # needs x86_64
    @skipIf(debug_info="gmodules")  # not relevant
    @skipIf(compiler="clang", compiler_version=["<", "7.0"])
    def testTargetVarExpr(self):
        self.build()
        lldbutil.run_to_name_breakpoint(self, "main")
        self.expect(
            "help target variable",
            substrs=[
                "--no-args",
                "--no-recognized-args",
                "--no-locals",
                "--show-globals",
            ],
            matching=False,
        )
        self.expect("target variable i", substrs=["i", "42"])
        self.expect(
            "target variable var", patterns=[r"\(incomplete \*\) var = 0[xX](0)*dead"]
        )
        self.expect(
            "target variable var[0]",
            error=True,
            substrs=["can't find global variable 'var[0]'"],
        )

        command_result = lldb.SBCommandReturnObject()
        result = self.ci.HandleCommand("target var", command_result)
        value_list = command_result.GetValues(lldb.eNoDynamicValues)
        self.assertGreaterEqual(value_list.GetSize(), 2)
        value_names = []
        for value in value_list:
            value_names.append(value.GetName())
        self.assertIn("i", value_names)



# RUN: %PYTHON %s | FileCheck %s
# This is just a smoke test that the dialect is functional.

from mlir.ir import *
from mlir.dialects import nvvm
from mlir.dialects import llvm
from mlir.dialects import func
import mlir.extras.types as T
from mlir.dialects import arith


def constructAndPrintInModule(f):
    print("\nTEST:", f.__name__)
    with Context(), Location.unknown():
        module = Module.create()
        with InsertionPoint(module.body):
            f()

        print(module)
        module.operation.verify()
    return f


# CHECK-LABEL: testSmoke
@constructAndPrintInModule
def testSmoke():
    i64 = IntegerType.get_signless(64)
    mat64f32_t = Type.parse(
        "!llvm.struct<(f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32, f32)>"
    )
    shape_attr = Attribute.parse("#nvvm.shape<m = 64, n = 32, k = 16>")

    # CHECK-LABEL: func @wgmma_f32_f16_f16(%arg0: i64, %arg1: i64)
    @func.FuncOp.from_py_func(i64, i64)
    def wgmma_f32_f16_f16(desc_a, desc_b):
        # CHECK: nvvm.cp.async.wait.group 5
        nvvm.CpAsyncWaitGroupOp(5)
        # CHECK: %0 = llvm.mlir.undef : [[MAT_T:.*]]
        result = llvm.UndefOp(mat64f32_t)
        # CHECK: %1 = nvvm.wgmma.mma_async %arg0, %arg1, %0, <m = 64, n = 32, k = 16>, D[<f32>, <zero>], A[<f16>, <neg>, <col>], B[<f16>, <neg>, <col>] : [[MAT_T]] -> [[MAT_T]]
        result1 = nvvm.WgmmaMmaAsyncOp(
            results_=mat64f32_t,
            inouts=result,
            descriptorA=desc_a,
            descriptorB=desc_b,
            shape=shape_attr,
            typeA=nvvm.WGMMATypes.f16,
            typeB=nvvm.WGMMATypes.f16,
            typeD=nvvm.WGMMATypes.f32,
            scaleD=nvvm.WGMMAScaleOut.zero,
            scaleA=nvvm.WGMMAScaleIn.neg,
            scaleB=nvvm.WGMMAScaleIn.neg,
            layoutA=nvvm.MMALayout.col,
            layoutB=nvvm.MMALayout.col,
        )


# CHECK-LABEL: TEST: test_inline_ptx
# CHECK-LABEL: func.func @my_inline_ptx(
# CHECK-SAME: %[[arg0:[a-zA-Z0-9_]+]]: f32, %[[arg1:[a-zA-Z0-9_]+]]: f32, %[[arg2:[a-zA-Z0-9_]+]]: i32, %[[arg3:[a-zA-Z0-9_]+]]: i32)
# CHECK: %[[S0:.+]]:2 = nvvm.inline_ptx
# CHECK-SAME: ro(%[[arg0]], %[[arg1]] : f32, f32) rw(%[[arg2]], %[[arg3]] : i32, i32) -> f32, f32
# CHECK: %[[S1:.+]] = arith.addf %[[arg0]], %[[arg1]] : f32
# CHECK: %[[S2:.+]] = arith.addi %[[arg2]], %[[arg3]] : i32
# CHECK: %[[S3:.+]] = arith.addf %[[S0]]#0, %[[S0]]#1 : f32


@constructAndPrintInModule
def test_inline_ptx():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(f32, f32, i32, i32)
    def my_inline_ptx(a, b, c, d):
        ptx = r"""
            {
                .reg .pred p;
                setp.ge.s32   p,      {$r0}, {$r1};
                selp.s32      {$r0},  {$r0}, {$r1}, p;
                selp.s32      {$r1},  {$r0}, {$r1}, p;
                selp.s32      {$rw0}, {$r0}, {$r1}, p;
                selp.s32      {$rw1}, {$r0}, {$r1}, p;
            }
            """
        wo0, wo1 = nvvm.inline_ptx(
            read_only_args=[a, b],
            read_write_args=[c, d],
            write_only_args=[f32, f32],
            ptx_code=ptx,
        )
        arith.addf(a, b)
        arith.addi(c, d)
        arith.addf(wo0, wo1)


@constructAndPrintInModule
def test_mbarrier_arrive():
    ptr_shared = llvm.PointerType.get(3)
    ptr_cluster = llvm.PointerType.get(7)
    i32 = T.i32()

    @func.FuncOp.from_py_func(ptr_shared, ptr_cluster, i32)
    def mbarrier_arrive_ops(barrier_shared, barrier_cluster, txcount):
        token = nvvm.mbarrier_arrive(barrier_shared)
        nvvm.mbarrier_arrive(barrier_cluster)
        token2 = nvvm.mbarrier_arrive_drop(barrier_shared)
        nvvm.mbarrier_arrive_drop(barrier_cluster)
        token3 = nvvm.mbarrier_arrive_expect_tx(barrier_shared, txcount)
        nvvm.mbarrier_arrive_expect_tx(barrier_cluster, txcount)
        token4 = nvvm.mbarrier_arrive_drop_expect_tx(barrier_shared, txcount)
        nvvm.mbarrier_arrive_drop_expect_tx(barrier_cluster, txcount)


# CHECK-LABEL:   func.func @mbarrier_arrive_ops(
# CHECK-SAME:      %[[SHARED:.*]]: !llvm.ptr<3>, %[[CLUSTER:.*]]: !llvm.ptr<7>, %[[TXCOUNT:.*]]: i32)
# CHECK:           %{{.*}} = nvvm.mbarrier.arrive %[[SHARED]] : !llvm.ptr<3> -> i64
# CHECK-NEXT:      nvvm.mbarrier.arrive %[[CLUSTER]] : !llvm.ptr<7>{{$}}
# CHECK-NEXT:      %{{.*}} = nvvm.mbarrier.arrive_drop %[[SHARED]] : !llvm.ptr<3> -> i64
# CHECK-NEXT:      nvvm.mbarrier.arrive_drop %[[CLUSTER]] : !llvm.ptr<7>{{$}}
# CHECK-NEXT:      %{{.*}} = nvvm.mbarrier.arrive.expect_tx %[[SHARED]], %[[TXCOUNT]] : !llvm.ptr<3>, i32 -> i64
# CHECK-NEXT:      nvvm.mbarrier.arrive.expect_tx %[[CLUSTER]], %[[TXCOUNT]] : !llvm.ptr<7>, i32{{$}}
# CHECK-NEXT:      %{{.*}} = nvvm.mbarrier.arrive_drop.expect_tx %[[SHARED]], %[[TXCOUNT]] : !llvm.ptr<3>, i32 -> i64
# CHECK-NEXT:      nvvm.mbarrier.arrive_drop.expect_tx %[[CLUSTER]], %[[TXCOUNT]] : !llvm.ptr<7>, i32{{$}}


@constructAndPrintInModule
def test_barriers():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(i32, i32, f32)
    def barriers(mask, vi32, vf32):
        c0 = arith.constant(T.i32(), 0)
        cffff = arith.constant(T.i32(), 0xFFFF)
        nvvm.barrier(
            barrier_id=c0,
            number_of_threads=cffff,
        )

        pred = arith.constant(T.i32(), 1)
        for reduction in (
            nvvm.BarrierReduction.AND,
            nvvm.BarrierReduction.OR,
            nvvm.BarrierReduction.POPC,
        ):
            pred = nvvm.barrier(
                reduction_op=reduction,
                reduction_predicate=pred,
            )

        nvvm.barrier0()
        nvvm.bar_warp_sync(mask)
        nvvm.cluster_arrive()
        nvvm.cluster_arrive(aligned=True)
        nvvm.cluster_arrive_relaxed()
        nvvm.cluster_arrive_relaxed(aligned=True)
        nvvm.cluster_wait()
        nvvm.cluster_wait(aligned=True)
        nvvm.fence_mbarrier_init()
        nvvm.bar_warp_sync(mask)
        return pred


# CHECK-LABEL:   func.func @barriers(
# CHECK:           %[[ARG0:.*]]: i32, %[[ARG1:.*]]: i32, %[[ARG2:.*]]: f32) -> i32 {
# CHECK:           %[[CONSTANT_0:.*]] = arith.constant 0 : i32
# CHECK:           %[[CONSTANT_1:.*]] = arith.constant 65535 : i32
# CHECK:           nvvm.barrier id = %[[CONSTANT_0]] number_of_threads = %[[CONSTANT_1]]
# CHECK:           %[[PRED:.*]] = arith.constant 1 : i32
# CHECK:           %[[BARRIER_1:.*]] = nvvm.barrier #nvvm.reduction<and> %[[PRED]] -> i32
# CHECK:           %[[BARRIER_2:.*]] = nvvm.barrier #nvvm.reduction<or> %[[BARRIER_1]] -> i32
# CHECK:           %[[BARRIER_3:.*]] = nvvm.barrier #nvvm.reduction<popc> %[[BARRIER_2]] -> i32
# CHECK:           nvvm.barrier0
# CHECK:           nvvm.bar.warp.sync %[[ARG0]] : i32
# CHECK:           nvvm.cluster.arrive
# CHECK:           nvvm.cluster.arrive {aligned}
# CHECK:           nvvm.cluster.arrive.relaxed
# CHECK:           nvvm.cluster.arrive.relaxed {aligned}
# CHECK:           nvvm.cluster.wait
# CHECK:           nvvm.cluster.wait {aligned}
# CHECK:           nvvm.fence.mbarrier.init
# CHECK:           nvvm.bar.warp.sync %[[ARG0]] : i32
# CHECK:           return %[[BARRIER_3]] : i32


@constructAndPrintInModule
def test_vote_sync_infer_type():
    i1 = IntegerType.get_signless(1)
    i32 = T.i32()

    @func.FuncOp.from_py_func(i32, i1)
    def vote_sync_ops(mask, pred):
        ballot_res = nvvm.vote_sync(mask, pred, nvvm.VoteSyncKind.ballot)
        any_res = nvvm.vote_sync(mask, pred, nvvm.VoteSyncKind.any)
        all_res = nvvm.vote_sync(mask, pred, nvvm.VoteSyncKind.all)
        uni_res = nvvm.vote_sync(mask, pred, nvvm.VoteSyncKind.uni)
        return ballot_res


# CHECK-LABEL:   func.func @vote_sync_ops(
# CHECK-SAME:      %[[MASK:.*]]: i32, %[[PRED:.*]]: i1) -> i32 {
# CHECK:           %[[BALLOT:.*]] = nvvm.vote.sync ballot %[[MASK]], %[[PRED]] -> i32
# CHECK:           %[[ANY:.*]] = nvvm.vote.sync any %[[MASK]], %[[PRED]] -> i1
# CHECK:           %[[ALL:.*]] = nvvm.vote.sync all %[[MASK]], %[[PRED]] -> i1
# CHECK:           %[[UNI:.*]] = nvvm.vote.sync uni %[[MASK]], %[[PRED]] -> i1
# CHECK:           return %[[BALLOT]] : i32


@constructAndPrintInModule
def test_clusterlaunchcontrol_query_cancel_infer_type():
    i128 = IntegerType.get_signless(128)

    @func.FuncOp.from_py_func(i128)
    def query_cancel_ops(response):
        is_canceled = nvvm.clusterlaunchcontrol_query_cancel(
            nvvm.ClusterLaunchControlQueryType.IS_CANCELED,
            response,
        )
        cta_x = nvvm.clusterlaunchcontrol_query_cancel(
            nvvm.ClusterLaunchControlQueryType.GET_FIRST_CTA_ID_X,
            response,
        )
        return cta_x


# CHECK-LABEL:   func.func @query_cancel_ops(
# CHECK-SAME:      %[[RESPONSE:.*]]: i128) -> i32 {
# CHECK:           %{{.*}} = nvvm.clusterlaunchcontrol.query.cancel query = is_canceled, %[[RESPONSE]] : i1
# CHECK:           %[[CTA_X:.*]] = nvvm.clusterlaunchcontrol.query.cancel query = get_first_cta_id_x, %[[RESPONSE]] : i32
# CHECK:           return %[[CTA_X]] : i32


@constructAndPrintInModule
def test_match_sync_infer_type():
    i32 = T.i32()
    i64 = IntegerType.get_signless(64)

    @func.FuncOp.from_py_func(i32, i32, i64)
    def match_sync_ops(mask, i32val, i64val):
        any_result = nvvm.match_sync(mask, i32val, nvvm.MatchSyncKind.any)
        all_result = nvvm.match_sync(mask, i32val, nvvm.MatchSyncKind.all)
        return any_result


# CHECK-LABEL:   func.func @match_sync_ops(
# CHECK-SAME:      %[[MASK:.*]]: i32, %[[I32VAL:.*]]: i32, %[[I64VAL:.*]]: i64) -> i32 {
# CHECK:           %[[ANY:.*]] = nvvm.match.sync any %[[MASK]], %[[I32VAL]] : i32 -> i32
# CHECK:           %[[ALL:.*]] = nvvm.match.sync all %[[MASK]], %[[I32VAL]] : i32 -> !llvm.struct<(i32, i1)>
# CHECK:           return %[[ANY]] : i32


@constructAndPrintInModule
def test_shfl_sync_infer_type():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(i32, i32, f32, i32, i32)
    def shfl_sync_ops(mask, i32val, f32val, offset, clamp):
        i32_result = nvvm.shfl_sync(mask, i32val, offset, clamp, nvvm.ShflKind.bfly)
        f32_result = nvvm.shfl_sync(mask, f32val, offset, clamp, nvvm.ShflKind.bfly)
        struct_result = nvvm.shfl_sync(
            mask,
            i32val,
            offset,
            clamp,
            nvvm.ShflKind.bfly,
            return_value_and_is_valid=True,
        )
        return i32_result


# CHECK-LABEL:   func.func @shfl_sync_ops(
# CHECK-SAME:      %[[MASK:.*]]: i32, %[[I32VAL:.*]]: i32, %[[F32VAL:.*]]: f32, %[[OFF:.*]]: i32, %[[CLAMP:.*]]: i32) -> i32 {
# CHECK:           %[[I32R:.*]] = nvvm.shfl.sync bfly %[[MASK]], %[[I32VAL]], %[[OFF]], %[[CLAMP]] : i32 -> i32
# CHECK:           %[[F32R:.*]] = nvvm.shfl.sync bfly %[[MASK]], %[[F32VAL]], %[[OFF]], %[[CLAMP]] : f32 -> f32
# CHECK:           %[[STRUCT:.*]] = nvvm.shfl.sync bfly %[[MASK]], %[[I32VAL]], %[[OFF]], %[[CLAMP]] {return_value_and_is_valid} : i32 -> !llvm.struct<(i32, i1)>
# CHECK:           return %[[I32R]] : i32


@constructAndPrintInModule
def test_ldmatrix_infer_type():
    ptr_shared = llvm.PointerType.get(3)

    shape_8x8 = Attribute.parse("#nvvm.ld_st_matrix_shape<m = 8, n = 8>")
    elt_b16 = Attribute.parse("#nvvm.ld_st_matrix_elt_type<b16>")

    @func.FuncOp.from_py_func(ptr_shared)
    def ldmatrix_ops(ptr):
        r1 = nvvm.ldmatrix(
            ptr,
            num=1,
            layout=nvvm.MMALayout.row,
            shape=shape_8x8,
            elt_type=elt_b16,
        )
        r4 = nvvm.ldmatrix(
            ptr,
            num=4,
            layout=nvvm.MMALayout.row,
            shape=shape_8x8,
            elt_type=elt_b16,
        )
        return r1


# CHECK-LABEL:   func.func @ldmatrix_ops(
# CHECK-SAME:      %[[PTR:.*]]: !llvm.ptr<3>) -> i32 {
# CHECK:           %[[R1:.*]] = nvvm.ldmatrix %[[PTR]] {{.*}}num = 1{{.*}} : (!llvm.ptr<3>) -> i32
# CHECK:           %[[R4:.*]] = nvvm.ldmatrix %[[PTR]] {{.*}}num = 4{{.*}} : (!llvm.ptr<3>) -> !llvm.struct<(i32, i32, i32, i32)>
# CHECK:           return %[[R1]] : i32


@constructAndPrintInModule
def test_reductions():
    i32 = T.i32()
    f32 = T.f32()

    @func.FuncOp.from_py_func(i32, i32, f32)
    def reductions(mask, vi32, vf32):
        for abs in (True, False):
            for nan in (True, False):
                for kind in (
                    nvvm.ReductionKind.AND,
                    nvvm.ReductionKind.MAX,
                    nvvm.ReductionKind.MIN,
                    nvvm.ReductionKind.OR,
                    nvvm.ReductionKind.UMAX,
                    nvvm.ReductionKind.UMIN,
                    nvvm.ReductionKind.XOR,
                ):
                    nvvm.redux_sync(vi32, kind, vi32)

                for kind in (
                    nvvm.ReductionKind.FMIN,
                    nvvm.ReductionKind.FMAX,
                ):
                    nvvm.redux_sync(vf32, kind, vi32, abs=abs, nan=nan)


# CHECK-LABEL:   func.func @reductions(
# CHECK:           %[[ARG0:.*]]: i32, %[[ARG1:.*]]: i32, %[[ARG2:.*]]: f32) {
# CHECK:           %[[REDUX_0:.*]] = nvvm.redux.sync and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_1:.*]] = nvvm.redux.sync max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_2:.*]] = nvvm.redux.sync min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_3:.*]] = nvvm.redux.sync or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_4:.*]] = nvvm.redux.sync umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_5:.*]] = nvvm.redux.sync umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_6:.*]] = nvvm.redux.sync xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_7:.*]] = nvvm.redux.sync fmin %[[ARG2]], %[[ARG1]] {abs = true, nan = true} : f32 -> f32
# CHECK:           %[[REDUX_8:.*]] = nvvm.redux.sync fmax %[[ARG2]], %[[ARG1]] {abs = true, nan = true} : f32 -> f32
# CHECK:           %[[REDUX_9:.*]] = nvvm.redux.sync and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_10:.*]] = nvvm.redux.sync max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_11:.*]] = nvvm.redux.sync min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_12:.*]] = nvvm.redux.sync or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_13:.*]] = nvvm.redux.sync umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_14:.*]] = nvvm.redux.sync umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_15:.*]] = nvvm.redux.sync xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_16:.*]] = nvvm.redux.sync fmin %[[ARG2]], %[[ARG1]] {abs = true} : f32 -> f32
# CHECK:           %[[REDUX_17:.*]] = nvvm.redux.sync fmax %[[ARG2]], %[[ARG1]] {abs = true} : f32 -> f32
# CHECK:           %[[REDUX_18:.*]] = nvvm.redux.sync and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_19:.*]] = nvvm.redux.sync max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_20:.*]] = nvvm.redux.sync min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_21:.*]] = nvvm.redux.sync or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_22:.*]] = nvvm.redux.sync umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_23:.*]] = nvvm.redux.sync umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_24:.*]] = nvvm.redux.sync xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_25:.*]] = nvvm.redux.sync fmin %[[ARG2]], %[[ARG1]] {nan = true} : f32 -> f32
# CHECK:           %[[REDUX_26:.*]] = nvvm.redux.sync fmax %[[ARG2]], %[[ARG1]] {nan = true} : f32 -> f32
# CHECK:           %[[REDUX_27:.*]] = nvvm.redux.sync and %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_28:.*]] = nvvm.redux.sync max %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_29:.*]] = nvvm.redux.sync min %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_30:.*]] = nvvm.redux.sync or %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_31:.*]] = nvvm.redux.sync umax %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_32:.*]] = nvvm.redux.sync umin %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_33:.*]] = nvvm.redux.sync xor %[[ARG1]], %[[ARG1]] : i32 -> i32
# CHECK:           %[[REDUX_34:.*]] = nvvm.redux.sync fmin %[[ARG2]], %[[ARG1]] : f32 -> f32
# CHECK:           %[[REDUX_35:.*]] = nvvm.redux.sync fmax %[[ARG2]], %[[ARG1]] : f32 -> f32
# CHECK:           return
# CHECK:         }



import lldb
import lldb.formatters
import lldb.formatters.synth


class SyntheticChildrenProvider(
    lldb.formatters.synth.PythonObjectSyntheticChildProvider
):
    def __init__(self, value, internal_dict):
        lldb.formatters.synth.PythonObjectSyntheticChildProvider.__init__(
            self, value, internal_dict
        )

    def make_children(self):
        return [("ID", 123456), ("Name", "Enrico"), ("Rate", 1.25)]



"""
Use lldb Python API to make sure the dynamic checkers are doing their jobs.
"""


import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class ObjCCheckerTestCase(TestBase):
    NO_DEBUG_INFO_TESTCASE = True

    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)

        # Find the line number to break for main.c.
        self.source_name = "main.m"

    @add_test_categories(["pyapi"])
    def test_objc_checker(self):
        """Test that checkers catch unrecognized selectors"""
        if self.getArchitecture() == "i386":
            self.skipTest("requires Objective-C 2.0 runtime")

        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target from the debugger.

        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Set up our breakpoints:

        main_bkpt = target.BreakpointCreateBySourceRegex(
            "Set a breakpoint here.", lldb.SBFileSpec(self.source_name)
        )
        self.assertTrue(
            main_bkpt and main_bkpt.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at the entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())

        self.assertState(process.GetState(), lldb.eStateStopped, PROCESS_STOPPED)

        threads = lldbutil.get_threads_stopped_at_breakpoint(process, main_bkpt)
        self.assertEqual(len(threads), 1)
        thread = threads[0]

        #
        #  The class Simple doesn't have a count method.  Make sure that we don't
        #  actually try to send count but catch it as an unrecognized selector.

        frame = thread.GetFrameAtIndex(0)
        expr_value = frame.EvaluateExpression("(int) [my_simple count]", False)
        expr_error = expr_value.GetError()

        self.assertTrue(expr_error.Fail())

        # Make sure the call produced no NSLog stdout.
        stdout = process.GetSTDOUT(100)
        self.assertTrue(stdout is None or (len(stdout) == 0))

        # Make sure the error is helpful:
        err_string = expr_error.GetCString()
        self.assertIn("selector", err_string)

        #
        # Check that we correctly insert the checker for an
        # ObjC method with the struct return convention.
        # Getting this wrong would cause us to call the checker
        # with the wrong arguments, and the checker would crash
        # So I'm just checking "expression runs successfully" here:
        #
        expr_value = frame.EvaluateExpression("[my_simple getBigStruct]", False)
        expr_error = expr_value.GetError()

        self.assertSuccess(expr_error)



import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class TestCase(TestBase):
    @skipUnlessDarwin
    def test(self):
        self.build()
        if self.TraceOn():
            self.expect("log enable -v lldb formatters")
        lldbutil.run_to_source_breakpoint(
            self, "break here", lldb.SBFileSpec("main.cpp")
        )
        self.expect("v x", substrs=["(MyOptional<int>) x = None"])
        self.expect("v y", substrs=["(MyOptional<int>) y = 42"])



"""
Test lldb-dap launch request.
"""

import lldbdap_testcase


class TestDAP_launch_failing_console(lldbdap_testcase.DAPTestCaseBase):
    """
    Tests launching in console with an invalid terminal type.
    """

    def test(self):
        program = self.getBuildArtifact("a.out")
        self.create_debug_adapter()
        launch_seq = self.launch(program, console="invalid")
        response = self.dap_server.receive_response(launch_seq)
        self.assertFalse(response["success"])
        self.assertTrue(self.get_dict_value(response, ["body", "error", "showUser"]))
        self.assertRegex(
            response["body"]["error"]["format"],
            r"unexpected value, expected 'internalConsole\', 'integratedTerminal\' or 'externalTerminal\' at arguments.console",
        )



"""
Test SBSymbolContext APIs.
"""

import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil


class SymbolContextAPITestCase(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number to of function 'c'.
        self.line = line_number(
            "main.c", '// Find the line number of function "c" here.'
        )

    @expectedFailureAll(oslist=["windows"], bugnumber="llvm.org/pr24778")
    def test(self):
        """Exercise SBSymbolContext API extensively."""
        self.build()
        exe = self.getBuildArtifact("a.out")

        # Create a target by the debugger.
        target = self.dbg.CreateTarget(exe)
        self.assertTrue(target, VALID_TARGET)

        # Now create a breakpoint on main.c by name 'c'.
        breakpoint = target.BreakpointCreateByName("c", exe)
        self.assertTrue(
            breakpoint and breakpoint.GetNumLocations() == 1, VALID_BREAKPOINT
        )

        # Now launch the process, and do not stop at entry point.
        process = target.LaunchSimple(None, None, self.get_process_working_directory())
        self.assertTrue(process, PROCESS_IS_VALID)

        # Frame #0 should be on self.line.
        thread = lldbutil.get_stopped_thread(process, lldb.eStopReasonBreakpoint)
        self.assertTrue(
            thread.IsValid(), "There should be a thread stopped due to breakpoint"
        )
        frame0 = thread.GetFrameAtIndex(0)
        self.assertEqual(frame0.GetLineEntry().GetLine(), self.line)

        # Now get the SBSymbolContext from this frame.  We want everything. :-)
        context = frame0.GetSymbolContext(lldb.eSymbolContextEverything)
        self.assertTrue(context)

        # Get the description of this module.
        module = context.GetModule()
        desc = lldbutil.get_description(module)
        self.expect(desc, "The module should match", exe=False, substrs=[exe])

        compileUnit = context.GetCompileUnit()
        self.expect(
            str(compileUnit),
            "The compile unit should match",
            exe=False,
            substrs=[self.getSourcePath("main.c")],
        )

        function = context.GetFunction()
        self.assertTrue(function)

        block = context.GetBlock()
        self.assertTrue(block)

        lineEntry = context.GetLineEntry()
        self.expect(
            lineEntry.GetFileSpec().GetDirectory(),
            "The line entry should have the correct directory",
            exe=False,
            substrs=[self.mydir],
        )
        self.expect(
            lineEntry.GetFileSpec().GetFilename(),
            "The line entry should have the correct filename",
            exe=False,
            substrs=["main.c"],
        )
        self.assertEqual(
            lineEntry.GetLine(), self.line, "The line entry's line number should match "
        )

        symbol = context.GetSymbol()
        self.assertTrue(
            function.GetName() == symbol.GetName() and symbol.GetName() == "c",
            "The symbol name should be 'c'",
        )

        sc_list = lldb.SBSymbolContextList()
        sc_list.Append(context)
        self.assertEqual(len(sc_list), 1)
        for sc in sc_list:
            self.assertEqual(lineEntry, sc.GetLineEntry())



"""Test the LLDB module cache funcionality for universal mach-o files."""

import glob
import lldb
from lldbsuite.test.decorators import *
from lldbsuite.test.lldbtest import *
from lldbsuite.test import lldbutil
import os
import time


class ModuleCacheTestcaseUniversal(TestBase):
    def setUp(self):
        # Call super's setUp().
        TestBase.setUp(self)
        # Find the line number in a(int) to break at.
        self.cache_dir = os.path.join(self.getBuildDir(), "lldb-module-cache")
        # Set the lldb module cache directory to a directory inside the build
        # artifacts directory so no other tests are interfered with.
        self.runCmd(
            'settings set symbols.lldb-index-cache-path "%s"' % (self.cache_dir)
        )
        self.runCmd("settings set symbols.enable-lldb-index-cache true")

    def get_module_cache_files(self, basename):
        module_file_glob = os.path.join(self.cache_dir, "llvmcache-*%s*" % (basename))
        return glob.glob(module_file_glob)

    # Doesn't depend on any specific debug information.
    @no_debug_info_test
    def test(self):
        """
        Test module cache functionality for a universal mach-o files.

        This will test that if we enable the module cache, we can create
        lldb module caches for each slice of a universal mach-o file and
        they will each have a unique directory.
        """
        exe_basename = "testit"
        src_dir = self.getSourceDir()
        yaml_path = os.path.join(src_dir, "universal.yaml")
        yaml_base, ext = os.path.splitext(yaml_path)
        exe = self.getBuildArtifact(exe_basename)
        self.yaml2obj(yaml_path, exe)
        self.assertTrue(os.path.exists(exe))
        # Create a module with no dependencies.
        self.runCmd("target create -d --arch x86_64 %s" % (exe))
        self.runCmd("image dump symtab %s" % (exe_basename))
        self.runCmd("target create -d --arch arm64 %s" % (exe))
        self.runCmd("image dump symtab %s" % (exe_basename))

        cache_files = self.get_module_cache_files(exe_basename)

        self.assertEqual(
            len(cache_files),
            2,
            "make sure there are two files in the module cache directory (%s) for %s"
            % (self.cache_dir, exe_basename),
        )
